// ===========================================================================
// OpenORB Examples
//
// (c) 2000, Exolab.org
// ===========================================================================

package dsi.boa;

public class DynSkeleton extends org.omg.CORBA.DynamicImplementation
{
	static final String[] _ids_list = 
	{
		"IDL:dsi/boa/Sample:1.0"
	};

	public String[ ] _ids()
	{
		return _ids_list;
	}
	
	private org.omg.CORBA.ORB orb;

	public DynSkeleton( org.omg.CORBA.ORB orb )
	{
		this.orb = orb;
	}

	public void invoke ( org.omg.CORBA.ServerRequest request )
	{
		// find which operation is being invoked.
		String operation = request.operation();

		if ( operation.equals("div") )
		{
			// create the argument list.
			org.omg.CORBA.NVList argList = orb.create_list(0);

			// first float argument
			org.omg.CORBA.Any arg0 = orb.create_any();
			arg0.type( orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_float) );
			argList.add_value("",arg0,org.omg.CORBA.ARG_IN.value);

			// second float argument
			org.omg.CORBA.Any arg1 = orb.create_any();
			arg1.type( orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_float) );
			argList.add_value("",arg1,org.omg.CORBA.ARG_IN.value);

			// set the arguments for the request. This will result in the arguments
			// being unmarshalled.
			request.arguments(argList);

			// extract the arguements
			float nb1= arg0.extract_float();
			float nb2 = arg1.extract_float();

			try
			{
				// can't divide by zero.
				if ( nb2 == 0 )
					throw new DivByZero();
				
				float resultat = nb1 / nb2 ;

				// create the result argument.
				org.omg.CORBA.Any any_result = orb.create_any();
				any_result.insert_float(resultat);
				
				// send the result
				request.set_result( any_result );

			}
			catch ( DivByZero ex )
			{
				// we throw an exception.
				org.omg.DynamicAny.DynAnyFactory factory = null;
				try
				{
					org.omg.CORBA.Object obj = null;
					obj = orb.resolve_initial_references("DynAnyFactory");
									
					factory =
					org.omg.DynamicAny.DynAnyFactoryHelper.narrow(obj);
				}
				catch ( org.omg.CORBA.ORBPackage.InvalidName e )
				{ }

				// The DivByZero exception has no arguments
				org.omg.CORBA.StructMember [] members = null;
				members = new org.omg.CORBA.StructMember[0];

				org.omg.CORBA.TypeCode tc = orb.create_exception_tc("IDL:dsi/boa/DivByZero:1.0","DivByZero",members);

				org.omg.DynamicAny.DynAny dany = null;

				try
				{
					dany = factory.create_dyn_any_from_type_code( tc );
				}
				catch ( org.omg.DynamicAny.DynAnyFactoryPackage.InconsistentTypeCode e )
				{ }

				org.omg.CORBA.Any any_ex = dany.to_any();
								
				request.set_exception(any_ex);
			}
		}
		else
			throw new org.omg.CORBA.BAD_OPERATION();
	}
}