/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Intalio Inc.  For written permission, please
 *    contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Intalio Inc. Exolab is a registered trademark of
 *    Intalio Inc.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY INTALIO AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL INTALIO OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2000 (C) Intalio Inc. All Rights Reserved.
 *
 * $Id: AnyTest.java,v 1.3 2001/08/13 03:47:34 wood Exp $
 *
 * Date         Author  Changes
 */
package org.openorb.test.CORBA;

import org.omg.PortableServer.POA;
import org.openorb.test.CORBA.CORBATestCase;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

public class AnyTest extends CORBATestCase {    
  public AnyTest(String name) {
    super( name );
  }
    
  public void setUp() {    
    super.setUp();

    _orb = getORB();
    _any = _orb.create_any();
  }
  
  private org.omg.CORBA.ORB _orb;
  private org.omg.CORBA.Any _any; 
  
  /**
   * Test the Any insert and extract operations. For each basic type (numbers, strings, objects) one invocation
   * is made for inserting an object into the Any, and the following invocation extracts the same value from the Any.
   * Checking is made that ensure with retrieve the same value;
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testAny() throws org.omg.CORBA.UserException {
    org.omg.CORBA.Any any2 = _orb.create_any();
    _any.insert_any( any2 );
    assertEquals("Values should be equal!", any2, _any.extract_any());

    _any.insert_char('c');
    assertEquals("Values should be equal!", 'c', _any.extract_char());

    _any.insert_wchar('c');
    assertEquals("Values should be equal!", 'c', _any.extract_wchar());

    _any.insert_octet((byte)'o');
    _any.extract_octet();

    _any.insert_short((short)1);
    _any.extract_short();

    _any.insert_ushort((short)1);
    _any.extract_ushort();

    _any.insert_long(2);
    _any.extract_long();

    _any.insert_ulong(2);
    _any.extract_ulong();

    _any.insert_longlong(2L);
    _any.extract_longlong();

    _any.insert_ulonglong(2L);
    _any.extract_ulonglong();

     _any.insert_float((float)3.0);
    _any.extract_float();

    _any.insert_double(3.0);
    _any.extract_double();

    _any.insert_string("str");
    _any.extract_string();

    _any.insert_wstring("str");
    _any.extract_wstring();

     _any.insert_Object(null);
    _any.extract_Object();

    org.openorb.CORBA.Principal p = new org.openorb.CORBA.Principal("MyPrincipal".getBytes());

    p.name();
    p.name("MyPrincipal".getBytes());
    assertTrue(  "Principal object not equal to self!", p.equals(p));

    _any.insert_Principal(p);
    _any.extract_Principal();
  }
   
  static public void main(String args[])
	 {
		  junit.textui.TestRunner.run(new TestSuite(AnyTest.class));
	 }
}