/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Intalio Inc.  For written permission, please
 *    contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Intalio Inc. Exolab is a registered trademark of
 *    Intalio Inc.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY INTALIO AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL INTALIO OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2000 (C) Intalio Inc. All Rights Reserved.
 *
 * $Id: CORBATestCase.java,v 1.2 2001/03/21 01:27:25 wood Exp $
 *
 * Date         Author  Changes
 */

package org.openorb.test.CORBA;

import java.io.*;
import junit.framework.TestCase;
import org.openorb.io.HexPrintStream;

import java.util.Properties;

/** 
 * Skeleton orb test case to use when testing an orb component. The pre_init and
 * post init functions deal with creating and destroying orb instances for the
 * client and server ends.
 *
 * @author <a href="mailto:wood@intalio.com">Chris Wood &lt;wood@intalio.com&gt;</a>
 * @version $Revision: 1.2 $ $Date: 2001/03/21 01:27:25 $ 
 */
public abstract class CORBATestCase extends TestCase 
{
	public CORBATestCase(String name) {
		super(name);
	}
	
  private org.omg.CORBA.ORB _orb;
  
  /**
   * This method is called prior to calling run and basically starts 
   * up a server and client orb, and spawns a thread for the server orb
   * to run with.
   */
	protected void setUp()
	{
    Properties props = new Properties();
    props.setProperty("openorb.useStaticThreadGroup", "true");
    props.setProperty("openorb.server.enable", "false");
    _orb = org.omg.CORBA.ORB.init((String[])null, props);
  }
  
  
  /**
   * This method is called after calling run. It shuts down the server and 
   * client orbs.
   */
	protected void tearDown()
	{
    _orb.shutdown(true);
	}  
  
	/**
	 * Get the server side orb.
	 */
  public org.omg.CORBA.ORB getORB() 
  {
    return _orb;
  }    
}