/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Intalio Inc.  For written permission, please
 *    contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Intalio Inc. Exolab is a registered trademark of
 *    Intalio Inc.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY INTALIO AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL INTALIO OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2000 (C) Intalio Inc. All Rights Reserved.
 *
 * $Id: DataStreamTest.java,v 1.2 2001/06/01 16:19:16 omodica Exp $
 *
 * Date         Author  Changes
 */
package org.openorb.test.CORBA;

import org.openorb.test.CORBA.CORBATestCase;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;
import org.omg.CORBA.ORB;
import org.omg.CORBA.Any;
import org.omg.CORBA.portable.InputStream;
import org.omg.CORBA.portable.OutputStream;
import org.omg.CORBA.DataInputStream;
import org.omg.CORBA.DataOutputStream;

public class DataStreamTest extends CORBATestCase {    
  public DataStreamTest(String name) {
    super( name );
  }
 
  public void setUp() {
    super.setUp();
    
    _orb = getORB();
    _any = _orb.create_any();

    _os = _orb.create_output_stream();
    _dos = new org.openorb.CORBA.DataOutputStream( _os );
  }
  
  private ORB _orb;
  private Any _any;
  private OutputStream _os;
  private DataOutputStream _dos;
  
  /**
   * Test the data output and input streams for simple objects. This operation
   * uses an output stream to write Anys, chars, numbers and strings. And an
   * input stream to read them back again.
   *
   * @exception Exception if any of the test case fails
   */
  public void testSimpleOutputStream() {
    _dos.write_any(_any);
    _dos.write_boolean(false);
    _dos.write_char('c');
    _dos.write_wchar('c');
    _dos.write_octet((byte)'o');
    _dos.write_short((short)1);
    _dos.write_ushort((short)1);
    _dos.write_long(2);
    _dos.write_ulong(2);
    _dos.write_longlong(3L);
    _dos.write_ulonglong(3L);
    _dos.write_float((float)4.0);
    _dos.write_double(4.0);
    _dos.write_string("str");
    _dos.write_wstring("str");
    _dos.write_Object(null);
    _dos.write_TypeCode(_orb.create_string_tc(10));
    
    InputStream is = _os.create_input_stream();
    DataInputStream dis = new org.openorb.CORBA.DataInputStream(is);
    
    dis.read_any();
    dis.read_boolean();
    dis.read_char();
    dis.read_wchar();
    dis.read_octet();
    dis.read_short();
    dis.read_ushort();
    dis.read_long();
    dis.read_ulong();
    dis.read_longlong();
    dis.read_ulonglong();
    dis.read_float();
    dis.read_double();
    dis.read_string();
    dis.read_wstring();
    dis.read_Object();
    dis.read_TypeCode();
  }
  
  /**
   * Test the data output stream for simple objects arrays. This operation uses an output stream
   * to write arrays of Anys, chars, numbers and strings.
   *
   * @exception Exception if any of the test case fails 
   */
  public void testArrayOutputStream() {
    _dos.write_any_array(new org.omg.CORBA.Any[]{_any, _any}, 0, 2 );
    _dos.write_boolean_array(new boolean[] {true, false}, 0, 2 );
    _dos.write_char_array(new char[] {'a', 'b', 'c'}, 0, 3 );
    _dos.write_wchar_array(new char[] {'a', 'b', 'c'}, 0, 3 );
    _dos.write_octet_array(new byte[] {(byte)'a', (byte)'b', (byte)'c'}, 0, 3 );
    _dos.write_short_array(new short[] {(short)0, (short)1, (short)2}, 0, 3 );
    _dos.write_ushort_array(new short[] {(short)0, (short)1, (short)2}, 0, 3 );
    _dos.write_long_array(new int[] {0, 1, 2}, 0, 3 );
    _dos.write_ulong_array(new int[] {0, 1, 2}, 0, 3 );
    _dos.write_longlong_array(new long[] {0L, 1L, 2L}, 0, 3 );
    _dos.write_ulonglong_array(new long[] {0L, 1L, 2L}, 0, 3 );
    _dos.write_float_array(new float[] {(float)0.1, (float)1.2, (float)2.3}, 0, 3 );
    _dos.write_double_array(new double[] {0.1, 1.2, 2.3}, 0, 3 );
    
    InputStream is = _os.create_input_stream();
    DataInputStream dis = new org.openorb.CORBA.DataInputStream(is);
    
    dis.read_any_array(new org.omg.CORBA.AnySeqHolder(new org.omg.CORBA.Any[2]), 0, 2 );
    dis.read_boolean_array(new org.omg.CORBA.BooleanSeqHolder(new boolean[2]), 0, 2 );
    dis.read_char_array(new org.omg.CORBA.CharSeqHolder(new char[3] ), 0, 3 );
    dis.read_wchar_array(new org.omg.CORBA.WCharSeqHolder(new char[3]), 0, 3 );
    dis.read_octet_array(new org.omg.CORBA.OctetSeqHolder(new byte[3]), 0, 3 );
    dis.read_short_array(new org.omg.CORBA.ShortSeqHolder(new short[3]), 0, 3 );
    dis.read_ushort_array(new org.omg.CORBA.UShortSeqHolder(new short[3]), 0, 3 );
    dis.read_long_array(new org.omg.CORBA.LongSeqHolder(new int[3]), 0, 3 );
    dis.read_ulong_array(new org.omg.CORBA.ULongSeqHolder(new int[3]), 0, 3 );
    dis.read_longlong_array(new org.omg.CORBA.LongLongSeqHolder(new long[3]), 0, 3 );
    dis.read_ulonglong_array(new org.omg.CORBA.ULongLongSeqHolder(new long[3]), 0, 3 );
    dis.read_float_array(new org.omg.CORBA.FloatSeqHolder(new float[3]), 0, 3 );
    dis.read_double_array(new org.omg.CORBA.DoubleSeqHolder(new double[3]), 0, 3 );
  }
   
  static public void main(String args[])
	 {
		  junit.textui.TestRunner.run(new TestSuite(DataStreamTest.class));
	 }
}