/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Intalio Inc.  For written permission, please
 *    contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Intalio Inc. Exolab is a registered trademark of
 *    Intalio Inc.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY INTALIO AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL INTALIO OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2000 (C) Intalio Inc. All Rights Reserved.
 *
 * $Id: ORBTest.java,v 1.3 2001/06/01 16:19:16 omodica Exp $
 *
 * Date         Author  Changes
 */
package org.openorb.test.CORBA;

import org.omg.PortableServer.POA;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;
import java.util.Properties;

public class ORBTest extends TestCase {    
  public ORBTest(String name) {
    super( name );
  }
    
  public void setUp() {
    Properties props = new Properties();
    props.setProperty("openorb.useStaticThreadGroup", "true");
    props.setProperty("openorb.server.enable", "false");
    _orb = org.omg.CORBA.ORB.init((String[])null, props);
    _any = _orb.create_any();
  }
  
	protected void tearDown()
	{
    _orb.shutdown(true);
	}  

  private org.omg.CORBA.ORB _orb;
  private org.omg.CORBA.Any _any;
    
  /**
   * ORBClass test. Test all ORB operations like resolve_initial_references
   * and create operations.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testORB() throws org.omg.CORBA.UserException {
    String[] list = _orb.list_initial_services();

    // resolve a local service
    _orb.resolve_initial_references("ORBPolicyManager");

    // resolve a nonlocal reference
    _orb.resolve_initial_references( "NameService" );

    // work ops.
    _orb.work_pending();

    _orb.perform_work();

    // factory ops
    _orb.create_list(3);

    _orb.create_named_value( "Test", null, 0);

    _orb.create_exception_list();

    _orb.create_context_list();

    _orb.create_environment();

    _orb.get_service_information( (short)0, null );

    /*
    _orb.send_multiple_requests_oneway ( new org.omg.CORBA.Request [0] );

    _orb.send_multiple_requests_deferred ( new org.omg.CORBA.Request [0] );

    _orb.poll_next_response();

    _orb.get_next_response();
    */
  }
   
  static public void main(String args[])
	{
		junit.textui.TestRunner.run(new TestSuite(ORBTest.class));
	}
}