/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Intalio Inc.  For written permission, please
 *    contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Intalio Inc. Exolab is a registered trademark of
 *    Intalio Inc.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY INTALIO AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL INTALIO OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2000 (C) Intalio Inc. All Rights Reserved.
 *
 * $Id: SingletonTest.java,v 1.2 2001/06/01 16:19:16 omodica Exp $
 *
 * Date         Author  Changes
 */
package org.openorb.test.CORBA;

import org.openorb.test.CORBA.CORBATestCase;
import org.omg.PortableServer.POA;
import org.openorb.test.ORBTestCase;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

public class SingletonTest extends CORBATestCase {    
  public SingletonTest(String name) {
    super( name );
  }
    
  public void setUp() {
    super.setUp();
    
    _orb = getORB();
    _singleton = (org.omg.CORBA_2_3.ORB)org.omg.CORBA.ORB.init();			
    _any = _orb.create_any();
  }
  
  private org.omg.CORBA.ORB _orb;
  private org.omg.CORBA_2_3.ORB _singleton;
  private org.omg.CORBA.Any _any;
  
  /**
   * Test the ReleaseInfo class which prints the OpenORB release information
   * such as the identification and version.
   *
   * @exception Exception if any of the test case fails
   */
  public void testReleaseInfo() {
    org.openorb.ReleaseInfo info = new org.openorb.ReleaseInfo();

    info.main( new String[0] );      
  }
  
  /**
   * Test the ORBSingleton operations to create TypeCodes. This test covers 
   * the create_XXX_tc operations passing by the right parameters.
   * The created TypeCodes are struct, union, enum, alias, exception, interface,
   * native, string, wstring, sequence, recursive, array, fixed, value_box, abstract interface,
   * local interface.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testTypeCodeCreate() throws org.omg.CORBA.UserException {
    _singleton.create_struct_tc( "IDL:Dummy:1.0", "ReallyDummy", new org.omg.CORBA.StructMember[0] );

    _singleton.create_union_tc( "IDL:Dummy:1.0", "ReallyDummy", _singleton.get_primitive_tc(org.omg.CORBA.TCKind.tk_short), new org.omg.CORBA.UnionMember[0] );

    _singleton.create_enum_tc(	"IDL:Dummy:1.0", "ReallyDummy", new	String[0] );

    _singleton.create_alias_tc(	"IDL:Dummy:1.0", "ReallyDummy", _singleton.create_string_tc(1) );

    _singleton.create_exception_tc( "IDL:Dummy:1.0", "ReallyDummy", new org.omg.CORBA.StructMember[0] );

    _singleton.create_interface_tc( "IDL:Dummy:1.0", "ReallyDummy" );

    _singleton.create_native_tc( "IDL:Dummy:1.0", "ReallyDummy" );

    _singleton.create_string_tc(1);

    _singleton.create_wstring_tc(1);

    _singleton.create_sequence_tc(3, _singleton.create_wstring_tc(1) );

    //_singleton.create_recursive_sequence_tc(3, 0 );

    _singleton.create_recursive_tc("IDL:Dummy:1.0");

    _singleton.create_array_tc(3, _singleton.create_wstring_tc(1) );

    _singleton.create_fixed_tc((short)5, (short)2 );

    _singleton.create_value_box_tc("IDL:Dummy:1.0", "ReallyDummy", _singleton.create_wstring_tc(1) );

    _singleton.create_abstract_interface_tc( "IDL:Dummy:1.0", "ReallyDummy" );

    ((org.openorb.CORBA.ORBSingleton)_singleton).create_local_interface_tc( "IDL:Dummy:1.0", "ReallyDummy" );
  }
  
 
  /**
   * Test the not implemented operations of the ORB Singleton. The whole not implemented operations
   * of the ORB Singleton (implement in the ORB) are invoked and the thrown NO_IMPLEMENT exception
   * is checked.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testORBSingletonNoImplement() throws org.omg.CORBA.UserException {
    try {
			String[] list = ((org.openorb.CORBA.ORBSingleton)_singleton).list_initial_services();
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }  
    try {
			_singleton.resolve_initial_references( "Dummy" );
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.run();
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.shutdown(false);
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.work_pending();
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.perform_work();
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.create_list(3);
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.create_named_value( "Test", null, 0);
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.create_exception_list();
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.create_context_list();
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.get_default_context();
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.create_environment();
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.connect(null);
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.disconnect(null);
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.create_output_stream();
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.object_to_string(null);
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.string_to_object("Dummy");
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.send_multiple_requests_oneway ( new org.omg.CORBA.Request [0] );
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.send_multiple_requests_deferred ( new org.omg.CORBA.Request [0] );
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.poll_next_response();
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.get_next_response();
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.register_value_factory ( "Dummy", null );
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.unregister_value_factory ( "Dummy" );
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.lookup_value_factory ( "Dummy" );
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.set_delegate ( null );
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.get_service_information( (short)0, null );
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
    try {
			_singleton.create_policy( 0, null );
			fail("NO_IMPLEMENT expected!");
    }
    catch( org.omg.CORBA.NO_IMPLEMENT ex ) {
			// expected
    }
  }
   
  static public void main(String args[])
	{
		junit.textui.TestRunner.run(new TestSuite(SingletonTest.class));
	}
}
