/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Intalio Inc.  For written permission, please
 *    contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Intalio Inc. Exolab is a registered trademark of
 *    Intalio Inc.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY INTALIO AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL INTALIO OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2000 (C) Intalio Inc. All Rights Reserved.
 *
 * $Id: TypeCodeTest.java,v 1.9 2001/08/13 03:47:34 wood Exp $
 *
 * Date         Author  Changes
 */
package org.openorb.test.CORBA;

import org.omg.PortableServer.POA;
import org.openorb.test.CORBA.CORBATestCase;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

public class TypeCodeTest extends CORBATestCase {    
  public TypeCodeTest( String name ) {
    super( name );
  }
    
  public void setUp() {
    super.setUp();

    _orb = getORB();
    _any = _orb.create_any();
  }
  
  private org.omg.CORBA.ORB _orb;
  private org.omg.CORBA.Any _any;
  
  /**
   * Test the primative TypeCodes operations. The test creates a short
   * primative TypeCode and then invokes all its operations.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testTCPrimative() throws org.omg.CORBA.UserException {
    org.omg.CORBA.TypeCode tc = _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_char);

    doTests(tc);      
  }
  
  
  /**
   * Test the string TypeCode operations. The test creates a string
   * TypeCode and then invokes all its operations.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testTCString() throws org.omg.CORBA.UserException {
    org.omg.CORBA.TypeCode tc = _orb.create_string_tc(1);

    doTests(tc);
  }
  
  /**
   * Test the alias TypeCode operations. The test creates a alias (aliased to a string)
   * TypeCode and then invokes all its operations.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testTCAlias() throws org.omg.CORBA.UserException {
    org.omg.CORBA.TypeCode tc = _orb.create_alias_tc(	"IDL:Dummy:1.0", "ReallyDummy", _orb.create_string_tc(1) );
      
    doTests(tc);
  }
  
  /**
   * Test the array TypeCode operations. The test creates a array (string elements)
   * TypeCode and then invokes all its operations.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testTCArray() throws org.omg.CORBA.UserException {
    org.omg.CORBA.TypeCode tc = _orb.create_array_tc(3, _orb.create_wstring_tc(1) );
      
    doTests(tc);
  }
  
  /**
   * Test the enum TypeCode operations. The test creates an enum
   * TypeCode and then invokes all its operations.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testTCEnum() throws org.omg.CORBA.UserException {
    org.omg.CORBA.TypeCode tc = _orb.create_enum_tc(	"IDL:Dummy:1.0", "ReallyDummy", new	String[0] );
      
    doTests(tc);
  }
    
  /**
   * Test the fixed TypeCode operations. The test creates a fixed
   * TypeCode and then invokes all its operations.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testTCFixed() throws org.omg.CORBA.UserException {
    org.omg.CORBA.TypeCode tc = _orb.create_fixed_tc((short)5, (short)2 );
      
    doTests(tc);     
  }
     
  /**
   * Test the object reference TypeCode operations. The test creates a object
   * reference TypeCode and then invokes all its operations.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testTCObject() throws org.omg.CORBA.UserException {
    org.omg.CORBA.TypeCode tc = _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_objref);
      
    doTests(tc);
  }
  
  /**
   * Test the struct TypeCode operations. The test creates a struct
   * TypeCode and then invokes all its operations.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testTCStruct() throws org.omg.CORBA.UserException {
    org.omg.CORBA.TypeCode tc = _orb.create_struct_tc( "IDL:Dummy:1.0", "ReallyDummy", new org.omg.CORBA.StructMember[0] );
      
    doTests(tc);
  }
    
  /**
   * Test the union TypeCode operations. The test creates an union
   * TypeCode and then invokes all its operations.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testTCUnion() throws org.omg.CORBA.UserException {
    org.omg.CORBA.TypeCode tc = _orb.create_union_tc( "IDL:Dummy:1.0", "ReallyDummy", _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_short), new org.omg.CORBA.UnionMember[0] );
      
    doTests(tc);
  }
  
  /**
   * Test the value TypeCode operations. The test creates a value
   * TypeCode and then invokes all its operations.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testTCValue() throws org.omg.CORBA.UserException {
    org.omg.CORBA.TypeCode tc = _orb.create_value_tc( "IDL:Dummy:1.0", "ReallyDummy", (short)0, _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_null), new org.omg.CORBA.ValueMember[0] );
      
    doTests(tc);
  }
  
  /**
   * Test the recursive TypeCode. This typecode must be embeded into another
   * typecode before being used.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails 
   */
  public void testTCRecursive() throws org.omg.CORBA.UserException {
    org.omg.CORBA.TypeCode tc = _orb.create_recursive_tc("IDL:Dummy:1.0");

    org.omg.CORBA.TypeCode seqTC = _orb.create_sequence_tc(1, tc);
    
    org.omg.CORBA.StructMember [] sm = new org.omg.CORBA.StructMember[] {
      new org.omg.CORBA.StructMember("backref", seqTC, null)
    };
    
    org.omg.CORBA.TypeCode strTC = _orb.create_struct_tc( "IDL:Dummy:1.0", "ReallyDummy", sm );

    assertTrue("Recursive typecode not properly formed",
                seqTC.content_type() == strTC);

    // do tests on the recursive typecode. These fail diferently to normal    
    tc.hashCode();
    tc.equals(tc);
    
    try {
      tc.equivalent(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_char));
      fail("expected exception");
    }
    catch(org.omg.CORBA.BAD_TYPECODE ex) {}
    try {
      tc.equal(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_char)); 
      fail("expected exception");
    }
    catch(org.omg.CORBA.BAD_TYPECODE ex) {}

    try {
      tc.get_compact_typecode();
      fail("expected exception");
    }
    catch(org.omg.CORBA.BAD_TYPECODE ex) {}

    // nonprimative types
    try {
      tc.id();
    }
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}

    try {
      tc.name();
      fail("expected exception");
    }
    catch(org.omg.CORBA.BAD_TYPECODE ex) {}
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}

    // strings, wstrings, arrays, sequences
    try {
      tc.length();
      fail("expected exception");
    }
    catch(org.omg.CORBA.BAD_TYPECODE ex) {}
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}

    // structs, exceptions, unions, enum, value
    try {
      tc.member_count();
      fail("expected exception");
    }
    catch(org.omg.CORBA.BAD_TYPECODE ex) {}
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}

    // structs, exceptions, unions, value
    try {
      tc.member_name(0);
      fail("expected exception");
    }
    catch(org.omg.CORBA.BAD_TYPECODE ex) {}
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}
    catch( org.omg.CORBA.TypeCodePackage.Bounds expected ) {}

    try {
      tc.member_type(0);
      fail("expected exception");
    }
    catch(org.omg.CORBA.BAD_TYPECODE ex) {}
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}
    catch( org.omg.CORBA.TypeCodePackage.Bounds expected ) {}

    // unions
    try {
      tc.member_label(0);
      fail("expected exception");
    }
    catch(org.omg.CORBA.BAD_TYPECODE ex) {}
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}
    catch( org.omg.CORBA.TypeCodePackage.Bounds expected ) {}

    try {
      tc.default_index();
      fail("expected exception");
    }
    catch(org.omg.CORBA.BAD_TYPECODE ex) {}
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}

    try {
      tc.discriminator_type();
      fail("expected exception");
    }
    catch(org.omg.CORBA.BAD_TYPECODE ex) {}
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}

    // sequences, arrays, aliases, boxed values
    try {
      tc.content_type();
      fail("expected exception");
    }
    catch(org.omg.CORBA.BAD_TYPECODE ex) {}
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}

    // fixed
    try {
      tc.fixed_digits();
      fail("expected exception");
    }
    catch(org.omg.CORBA.BAD_TYPECODE ex) {}
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}

    try {
      tc.fixed_scale();
      fail("expected exception");
    }
    catch(org.omg.CORBA.BAD_TYPECODE ex) {}
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}

    // value
    try {
      tc.type_modifier();
      fail("expected exception");
    }
    catch(org.omg.CORBA.BAD_TYPECODE ex) {}
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}

    try {
      tc.member_visibility(0);
      fail("expected exception");
    }
    catch(org.omg.CORBA.BAD_TYPECODE ex) {}
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}
    
    try {
      tc.concrete_base_type();
      fail("expected exception");
    }
    catch(org.omg.CORBA.BAD_TYPECODE ex) {}
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}   
  }  
  
  /**
   * Test all TypeCode operations.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  private void doTests( org.omg.CORBA.TypeCode tc ) throws org.omg.CORBA.UserException {
    tc.equivalent(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_char));
    tc.equal(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_char)); 

    tc.get_compact_typecode();

    tc.hashCode();

    // nonprimative types
    try {
      tc.id();
    }
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}

    try {
      tc.name();
    }
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}

    // strings, wstrings, arrays, sequences
    try {
      tc.length();
    }
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}

    // structs, exceptions, unions, enum, value
    try {
      tc.member_count();
    }
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}

    // structs, exceptions, unions, value
    try {
      tc.member_name(0);
    }
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}
    catch( org.omg.CORBA.TypeCodePackage.Bounds expected ) {}

    try {
      tc.member_type(0);
    }
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}
    catch( org.omg.CORBA.TypeCodePackage.Bounds expected ) {}

    // unions
    try {
      tc.member_label(0);
    }
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}
    catch( org.omg.CORBA.TypeCodePackage.Bounds expected ) {}

    try {
      tc.default_index();
    }
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}

    try {
      tc.discriminator_type();
    }
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}

    // sequences, arrays, aliases, boxed values
    try {
      tc.content_type();
    }
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}

    // fixed
    try {
      tc.fixed_digits();
    }
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}

    try {
      tc.fixed_scale();
    }
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}

    // value
    try {
      tc.type_modifier();
    }
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}

    try {
      tc.member_visibility(0);
      // needed in jdk 1.2
      if(false)
        throw new org.omg.CORBA.TypeCodePackage.Bounds();
    }
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}
    catch( org.omg.CORBA.TypeCodePackage.Bounds expected ) {}
    
    try {
      tc.concrete_base_type();
    }
    catch( org.omg.CORBA.TypeCodePackage.BadKind expected ) {}
  } 
    
  static public void main(String args[])
  {
    junit.textui.TestRunner.run(new TestSuite(TypeCodeTest.class));
  }
}