/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Intalio Inc.  For written permission, please
 *    contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Intalio Inc. Exolab is a registered trademark of
 *    Intalio Inc.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY INTALIO AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL INTALIO OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2000 (C) Intalio Inc. All Rights Reserved.
 *
 * $Id: ORBTestCase.java,v 1.5 2001/08/13 03:47:34 wood Exp $
 *
 * Date         Author  Changes
 */

package org.openorb.test;

import java.io.*;
import junit.framework.TestCase;
import org.openorb.io.HexPrintStream;

import java.util.Properties;

/** 
 * Skeleton orb test case to use when testing an orb component. The pre_init and
 * post init functions deal with creating and destroying orb instances for the
 * client and server ends.
 *
 * @author <a href="mailto:wood@intalio.com">Chris Wood &lt;wood@intalio.com&gt;</a>
 * @version $Revision: 1.5 $ $Date: 2001/08/13 03:47:34 $ 
 */
abstract public class ORBTestCase 
  extends TestCase 
{
	public ORBTestCase(String name) {
		super(name);
	}
	
  /**
   * This method is called prior to calling run and basically starts 
   * up a server and client orb, and spawns a thread for the server orb
   * to run with.
   */
	protected void setUp()
	{
		setUp(null);
	}
	
  /**
   * Override setUp and call this method with alternative properties to
	 * startup an orb with alternative parameters.
   */
	protected void setUp(Properties props) {
    if(props == null)
      _props = new Properties();
    else
  		_props = props;
    _props.setProperty("openorb.useStaticThreadGroup", "true");

		
    String [] args = new String[0];
		
    _serverORB = org.omg.CORBA.ORB.init(args, _props);
    
    _serverThread = new Thread(new Runnable() {
      public void run()
      {
        try {
          _serverORB.run();
        }
        catch( java.lang.Exception ex){}
      }
    });
    _serverThread.start();
  }
  
  
  /**
   * This method is called after calling run. It shuts down the server and 
   * client orbs.
   */
	protected void tearDown()
	{
    _serverORB.shutdown(true);
		_serverORB = null;
    
    try {
      _serverThread.join(20000);
    }
    catch(InterruptedException ex) {}
    
		assertTrue("ORBTestCase.postExecute: Unable to stop server orb", !_serverThread.isAlive());
		
		_props = null;
	}
  
	/**
	 * Restarts the server side orb.
	 */
  protected org.omg.CORBA.ORB restartORB() {
    _serverORB.shutdown(true);
    
    try {
      _serverThread.join(20000);
    }
    catch(InterruptedException ex) {}
    
    assertTrue("ORBTestCase.restartORB: Unable to stop server orb", !_serverThread.isAlive());
    
    String [] args = new String[0];

    _serverORB = org.omg.CORBA.ORB.init(args, _props);
        
    _serverThread = new Thread(new Runnable() {
      public void run()
      {
         _serverORB.run();
      }
    });
    _serverThread.start();
    
    return _serverORB;
  }
  
	/**
	 * Get the server side orb.
	 */
  public org.omg.CORBA.ORB getORB() 
  {
    return _serverORB;
  }
  	
	/**
	 * Sets local invoke policy on target.
	 */
  public org.omg.CORBA.Object forceMarshal(org.omg.CORBA.Object obj) throws org.omg.CORBA.PolicyError
  {
    if(_forcePolicy == null)
      _forcePolicy = _serverORB.create_policy(org.openorb.policy.FORCE_MARSHAL_POLICY_ID.value, _serverORB.create_any());
      
    return obj._set_policy_override(new org.omg.CORBA.Policy[] { _forcePolicy }, org.omg.CORBA.SetOverrideType.ADD_OVERRIDE);
  }
  
	/**
	 * Write buffer as hex to given stream.
	 */
  public static void writeVerboseHex(java.io.OutputStream stream, byte [] buf) 
    throws java.io.IOException
  {
    HexPrintStream hps = new HexPrintStream(stream, HexPrintStream.FORMAT_MIXED);    

    hps.write(buf);
    hps.flush();
	}
  
  private org.omg.CORBA.Policy _forcePolicy;
  private org.omg.CORBA.ORB _serverORB;
  private Thread _serverThread;
	private Properties _props;
}