/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Intalio Inc.  For written permission, please
 *    contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Intalio Inc. Exolab is a registered trademark of
 *    Intalio Inc.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY INTALIO AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL INTALIO OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2000 (C) Intalio Inc. All Rights Reserved.
 *
 * $Id: FWDTest.java,v 1.2 2001/08/13 03:47:34 wood Exp $
 *
 * Date         Author  Changes
 */

package org.openorb.test.adapter.fwd;

import org.omg.CORBA.Policy;
import org.omg.PortableServer.*;
import org.openorb.test.ORBTestCase;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

import java.util.Arrays;
import java.util.Properties;

import org.openorb.corbaloc.CorbalocService;
import org.openorb.corbaloc.CorbalocServiceHelper;

import org.openorb.test.adapter.poa.Hello;
import org.openorb.test.adapter.poa.HelloPOA;
import org.openorb.test.adapter.poa.HelloHelper;

/**
 * Tests range of POA policies and features.
 * <p>
 *
 * @version $Revision: 1.2 $ $Date: 2001/08/13 03:47:34 $ 
 * @author <a href="mailto:wood@intalio.com">Chris Wood &lt;wood@intalio.com&gt;</a>
 * @see     org.exolab.jtf.CWTestCateogry
 * @see     org.exolab.jtf.CWTestCase
 */
public class FWDTest extends ORBTestCase
{
	public FWDTest(String name) {
		super(name);
	}

  private static final String PORT = "23412";
  
  /**
   * Overloaded to allow the persistant test case to work.
   */
	protected void setUp()
	{
		java.util.Properties props = new java.util.Properties();
		// set known iiop port since persistant references are used.
    props.setProperty("ImportModule.Corbaloc", "${openorb.home}config/default.xml#CorbalocService");
    props.setProperty("iiop.port", PORT);
		setUp(props);
	}
	
  /**
   * Test that init reference service can be resolved.
   */
	public void testResolve() throws org.omg.CORBA.UserException {
    org.omg.CORBA.ORB orb = getORB();
    
    CorbalocService service = CorbalocServiceHelper.narrow(orb.resolve_initial_references("CorbalocService"));
    
    Hello hello = new HelloImpl()._this(getORB());
    
    service.put("Test", hello);

    assertTrue("Get object is not equivalent", hello._is_equivalent(service.get("Test")));
    
    org.omg.CORBA.Object obj = orb.string_to_object("corbaloc::localhost:" + PORT + "/Test");
    
    Hello ini = HelloHelper.narrow(obj);
    
    ini.hello("Mouse");
  }
  
  /**
   * Test that init reference service can be resolved.
   */
	public void testLocalCorbaloc() throws org.omg.CORBA.UserException {
    org.omg.CORBA.ORB orb = getORB();
    
    CorbalocService service = CorbalocServiceHelper.narrow(orb.string_to_object("corbaloc::localhost:" + PORT + "/CorbalocService"));
    
    Hello hello = new HelloImpl()._this(getORB());
    
    service.put("Test", hello);

    assertTrue("Get object is not equivalent", hello._is_equivalent(service.get("Test")));
    
    org.omg.CORBA.Object obj = orb.string_to_object("corbaloc::localhost:" + PORT + "/Test");
    
    Hello ini = HelloHelper.narrow(obj);
    
    ini.hello("Mouse");
  }
  
  /**
   * Test that init reference service can be resolved.
   */
	public void testRemoteCorbaloc() throws org.omg.CORBA.UserException {
    org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init((String[])null, null);
    Thread.currentThread().interrupt();
    orb.run();
    Thread.interrupted();
    
    CorbalocService service = CorbalocServiceHelper.narrow(orb.string_to_object("corbaloc::localhost:" + PORT + "/CorbalocService"));
    
    HelloImpl helloImpl = new HelloImpl();
    Hello hello = helloImpl._this(getORB());
    helloImpl._default_POA().the_POAManager().activate();
    
    service.put("Test", hello);

    assertTrue("Get object is not equivalent", hello._is_equivalent(service.get("Test")));
    
    org.omg.CORBA.Object obj = orb.string_to_object("corbaloc::localhost:" + PORT + "/Test");
    
    Hello ini = HelloHelper.narrow(obj);
    
    ini.hello("Mouse");
    
    
    orb.shutdown(true);
  }
  
	/**
	 *  Utility classes.
	 */
  static class HelloImpl 
    extends HelloPOA
  {
    public void hello(String msg) {
    }
  }
  
  static public void main(String args[])
	{
		junit.textui.TestRunner.run(new TestSuite(FWDTest.class));
	}
}