/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Intalio Inc.  For written permission, please
 *    contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Intalio Inc. Exolab is a registered trademark of
 *    Intalio Inc.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY INTALIO AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL INTALIO OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2000 (C) Intalio Inc. All Rights Reserved.
 *
 * $Id: DIITest.java,v 1.6 2001/08/13 03:47:34 wood Exp $
 *
 * Date         Author  Changes
 */
package org.openorb.test.dynamic;

import org.omg.PortableServer.POA;
import org.openorb.test.ORBTestCase;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

public class DIITest extends ORBTestCase {    
  public DIITest( String name ) {
    super( name );
  }
    
  public void setUp() {
    super.setUp();

		try {
			_orb = getORB();

			POA rootPOA = (POA)_orb.resolve_initial_references("RootPOA");

			DIITarget svr_ref = (new DIITargetImpl(rootPOA))._this(_orb);

			rootPOA.the_POAManager().activate();

			_cltRef = forceMarshal(svr_ref);

			_any = _orb.create_any();
		}
    catch(org.omg.CORBA.UserException ex) {
      fail("exception during setup:" + ex.toString());
    }
  }
  
  private org.omg.CORBA.ORB _orb;
  private org.omg.CORBA.Object _cltRef;
  private org.omg.CORBA.Any _any;
  
  /**
   * Test a simple invocation using the DII. The request is created with no parameter
   * and no result or exception are expected.
   *
   */
  public void testSimpleInvocation() {
    org.omg.CORBA.Request req = _cltRef._request("hello");

    req.invoke();
    
    java.lang.Exception exception = req.env().exception();
    req.env().clear();
    
    if(exception != null)
      fail( exception.getMessage() );
  }
    
  /**
   * Test a simple invocation using the DII with a return parameter. The request is created with no param
   * and a string result is expected. Trying to invoke the operation using the oneway mechanism 
   * throws an exception as expected.
   */
  public void testReturnInvocation() {
    org.omg.CORBA.Request req = _cltRef._request("message");
    
    org.omg.CORBA.TypeCode tc_return = _orb.get_primitive_tc( org.omg.CORBA.TCKind.tk_string );
          
    req.set_return_type( tc_return );
          
    req.invoke();

    java.lang.Exception exception = req.env().exception();
    if(exception != null)
      fail( exception.getMessage() );
    
    org.omg.CORBA.Any result = req.return_value();

    String msg = result.extract_string( );
    
    req.operation();
    
    req.arguments();
    
    req.result();
    
    req.target();
    
    req.ctx();    
  }
    
  /**
   * Test a simple invocation using the DII with a return parameter. The request is created with 2 float
   * parameters. A float result is expected. Typing of the result is checked.
   */
  public void testParamInvocation() {
    org.omg.CORBA.Request req = _cltRef._request("add");

    org.omg.CORBA.Any param1 = req.add_in_arg( );
    org.omg.CORBA.Any param2 = req.add_in_arg();

    param1.insert_float( 5 );
    param2.insert_float( ( float ) 3.14 );

    org.omg.CORBA.TypeCode tc_return = _orb.get_primitive_tc( org.omg.CORBA.TCKind.tk_float );

    req.set_return_type( tc_return );

    req.invoke();

    java.lang.Exception exception = req.env().exception();
    
    if(exception != null)
      fail( exception.getMessage() );
    
    org.omg.CORBA.Any result = req.return_value();

    float res = result.extract_float( );
  }
  
  /**
   * Test a simple invocation using the DII with an exception. The request is created with 2 float
   * parameters. A float result is expected. One invocation is made resulting in throwing an
   * an exception and another one without the exception. This method tests both cases.
   *
   * @exception org.omg.CORBA.TypeCodePackage.BadKind if any of the test cases fails
   */
  public void testExceptionInvocation() throws org.omg.CORBA.TypeCodePackage.BadKind {
    org.omg.CORBA.Request req = _cltRef._request("divide");

    org.omg.CORBA.Any param1 = req.add_in_arg( );
    org.omg.CORBA.Any param2 = req.add_in_arg();

    param1.insert_float( 5 );
    param2.insert_float( 0 );

    org.omg.CORBA.TypeCode tc_return = _orb.get_primitive_tc( org.omg.CORBA.TCKind.tk_float );

    req.set_return_type( tc_return );

    org.omg.CORBA.StructMember [ ] members = new org.omg.CORBA.StructMember[ 0 ];

    org.omg.CORBA.TypeCode tc_exception =
        _orb.create_exception_tc("IDL:openorb.org/test/dynamic/DIITarget/DivideByZero:1.0","DivideByZero", members );

    req.exceptions().add( tc_exception );

    req.invoke();

    org.omg.CORBA.Any result = req.return_value();

    java.lang.Exception exception = req.env().exception();
    assertNotNull( "Expected exception return", exception  );
    if (exception instanceof org.omg.CORBA.UnknownUserException ) {
      org.omg.CORBA.UnknownUserException unk_except = (org.omg.CORBA.UnknownUserException)exception;

      assertTrue("Unknown exception instead of DivideByZero exception",
        unk_except.except.type().id().equals("IDL:openorb.org/test/dynamic/DIITarget/DivideByZero:1.0") );
    }
    else {
      throw (org.omg.CORBA.SystemException) exception;
    }
  }    
     
  /**
   * Test a invocation using the DII with a context and context operations. The request is created with a test context
   * that was described in the IDL. Several operations are performed on the context, such as setting its value, creating
   * a child context and finally invoking the contructed request.
   */
  public void testContextInvocation() {
    org.omg.CORBA.Request req = _cltRef._request("clauseContext");

    req.contexts().add("Testing");

    org.omg.CORBA.Context aContext = _orb.get_default_context( );

    org.omg.CORBA.Any valueCtx = _orb.create_any( );

    valueCtx.insert_string("Context value for context 'Testing'. ");

    aContext.set_one_value( "Testing", valueCtx );

    aContext.set_values( _orb.create_list(2) );

    aContext.context_name();
    org.omg.CORBA.Context parentCtx = aContext.parent();
    org.omg.CORBA.Context childCtx = aContext.create_child( "ChildContext" );
    
    aContext.delete_values("None");
    
    req.ctx( aContext );

    req.invoke();

    java.lang.Exception exception = req.env().exception();
    
    if(exception != null)
      fail( exception.getMessage() );
  }
    
  /**
   * Test an attribute invocation using the DII. As an attribute was declared in the IDL, the request is created
   * and invoked for the _get_XXX and _set_XXX generated operations. Tests are performed to check good behaviour
   * of these operations.
   */
  public void testAttributeInvocation() {
    org.omg.CORBA.Request req_read = _cltRef._request("_get_name");

    org.omg.CORBA.Request req_write = _cltRef._request("_set_name");

    org.omg.CORBA.TypeCode tc_string = _orb.get_primitive_tc( org.omg.CORBA.TCKind.tk_string );

    req_read.set_return_type( tc_string );

    req_read.invoke();

    org.omg.CORBA.Any result = req_read.return_value();

    result.extract_string();

    org.omg.CORBA.Any param = req_write.add_in_arg();

    param.insert_string("NewName");

    req_write.invoke( );
  }
   
    
  /**
   * Test a oneway invocation.
   */
  public void testOnewayInvocation() {
    org.omg.CORBA.Request req = _cltRef._request("hello_oneway");

    req.send_oneway();
    
    java.lang.Exception exception = req.env().exception();
    req.env().clear();
    
    if(exception != null)
      fail( exception.getMessage() );
  }
  
  /**
   * Test sending multiple oneway requests.
   */
  public void testMultipleOnewayInvocation() {
    org.omg.CORBA.Request [] req = new org.omg.CORBA.Request[5];
    for(int i = 0; i < req.length; ++i)
      req[i] = _cltRef._request("hello_oneway");
    
    _orb.send_multiple_requests_oneway(req);
    
    for(int i = 0; i < req.length; ++i) {
      java.lang.Exception exception = req[i].env().exception();
      req[i].env().clear();
      
      if(exception != null)
        fail( exception.getMessage() );
    }
  }
    
  /**
   * Test a deferred invocation using the DII. A deferred request may be
   * delayed by the ORB and the result is available later. This test invokes
   * a request as delayed, and then gets the response using the get_response
   * function.
   *
   * @exception org.omg.CORBA.UserException if any of the test cases fails
   */
  public void testDeferredInvocation() throws org.omg.CORBA.UserException {
    org.omg.CORBA.Request req = _cltRef._request("message");

    org.omg.CORBA.TypeCode tc_return = _orb.get_primitive_tc( org.omg.CORBA.TCKind.tk_string );

    req.set_return_type( tc_return );

    req.send_deferred();

    req.get_response();

    org.omg.CORBA.Any result = req.return_value();

    String msg = result.extract_string( );
  }
  
  /**
   * Test a deferred invocation using the DII. A deferred request may be
   * delayed by the ORB and the result is available later. This test invokes
   * a request as delayed, polls until a response arrives and then gets the
   * response using the get_response function.
   *
   * @exception org.omg.CORBA.UserException if any of the test cases fails
   */
  public void testPollingInvocation() throws org.omg.CORBA.UserException {
    org.omg.CORBA.Request req = _cltRef._request("message");

    org.omg.CORBA.TypeCode tc_return = _orb.get_primitive_tc( org.omg.CORBA.TCKind.tk_string );

    req.set_return_type( tc_return );

    req.send_deferred();

    try {
      while(!req.poll_response())
        Thread.sleep(500);
    }
    catch(InterruptedException ex) {}
    
    req.get_response();

    org.omg.CORBA.Any result = req.return_value();

    String msg = result.extract_string( );
  }
  
  private org.omg.CORBA.Request crossThreadReq;
  private RuntimeException crossThreadException;
  
  /**
   * Test a deferred invocation using the DII. A deferred request may be
   * delayed by the ORB and the result is available later. This test invokes
   * a request as delayed, and then gets the response using the get_response
   * function in a different thread.
   *
   * @exception org.omg.CORBA.UserException if any of the test cases fails
   */
  public void testDeferredCrossThreadInvocation() throws org.omg.CORBA.UserException {
    crossThreadReq = _cltRef._request("message");

    org.omg.CORBA.TypeCode tc_return = _orb.get_primitive_tc( org.omg.CORBA.TCKind.tk_string );

    crossThreadReq.set_return_type( tc_return );

    crossThreadReq.send_deferred();

    Thread poll = new Thread(new Runnable() {
      public void run() {
        try {
          while(!crossThreadReq.poll_response())
            Thread.sleep(500);
        }
        catch(RuntimeException ex) {
          crossThreadException = ex;
        }
        catch(InterruptedException ex) {}
      }
    });
    
    poll.start();
    try {
      poll.join();
    }
    catch(InterruptedException ex) {}
    
    
    if(crossThreadException != null)
      throw crossThreadException;
    
    Thread getResponse = new Thread(new Runnable() {
      public void run() {
        try {
          crossThreadReq.get_response();
        }
        catch(RuntimeException ex) {
          crossThreadException = ex;
        }
        catch(Exception ex)
        { }
      }
    });
    
    getResponse.start();
    try {
      poll.join();
    }
    catch(InterruptedException ex) {}

    if(crossThreadException != null)
      throw crossThreadException;
    
    org.omg.CORBA.Any result = crossThreadReq.return_value();

    String msg = result.extract_string( );
  }
  
  /**
   * Test sending multiple defered requests, waiting for responses to arrive,
   * and recieving each response.
   *
   * @exception org.omg.CORBA.UserException if any of the test cases fails
   */
  public void testMultipleDeferredInvocations() throws org.omg.CORBA.UserException {
    org.omg.CORBA.Request [] req = new org.omg.CORBA.Request[5];
    
    org.omg.CORBA.TypeCode tc_return = _orb.get_primitive_tc( org.omg.CORBA.TCKind.tk_string );
    for(int i = 0; i < req.length; ++i) {
      req[i] = _cltRef._request("message");
      req[i].set_return_type( tc_return );
    }
    
    _orb.send_multiple_requests_deferred(req);
    
    for(int i = 0; i < req.length; ++i) {
      try {
        while(!_orb.poll_next_response())
          Thread.sleep(500);
      }
      catch(InterruptedException ex) {}
      
      org.omg.CORBA.Request next = _orb.get_next_response();
      
      org.omg.CORBA.Any result = next.return_value();

      String msg = result.extract_string( );
    }
  }
  
  /**
   * Servant implementation used for tests.
   */
  public static class DIITargetImpl extends DIITargetPOA {
    private String _name = "TargetRange";

    public DIITargetImpl( org.omg.PortableServer.POA poa ) {}

    public void hello() {}

    public void hello_oneway() {}
    
    public String message() {
      return "Hello from the server";
    }

    public String name() {
      return _name;
    }

    public void name( String n ) {
      _name = n;
    }

    public float add( float nb1, float nb2 ) {
      return nb1 + nb2;
    }

    public float divide( float nb1, float nb2 )
      throws org.openorb.test.dynamic.DIITargetPackage.DivideByZero
    {
      if ( nb2 == 0 )
        throw new org.openorb.test.dynamic.DIITargetPackage.DivideByZero();

      return nb1 / nb2;
    }

    public void clauseContext( org.omg.CORBA.Context ctx ) {
      org.omg.CORBA.NVList nv = null;
      try
      {
        nv = ctx.get_values("", 0, "Testing");
      }
      catch ( org.omg.CORBA.BAD_CONTEXT ex )
      { 
        return;
      }

      try
      {
        org.omg.CORBA.NamedValue n = nv.item(0);
        org.omg.CORBA.Any any = n.value();

        any.extract_string();
      }
      catch ( org.omg.CORBA.Bounds ex )
      {
        //System.out.println("No value found");
      }
    }	
    
    public String [] sequenceTest( String [] sequence )
    {
      return sequence;
    }
    
    public org.openorb.test.dynamic.Person structTest( org.openorb.test.dynamic.Person p )
    {
      return p;
    }
  }
  
    static public void main(String args[])
	{
		junit.textui.TestRunner.run(new TestSuite(DIITest.class));
	}
}