/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Intalio Inc.  For written permission, please
 *    contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Intalio Inc. Exolab is a registered trademark of
 *    Intalio Inc.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY INTALIO AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL INTALIO OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2000 (C) Intalio Inc. All Rights Reserved.
 *
 * $Id: DynAnyTest.java,v 1.2 2001/06/01 16:19:17 omodica Exp $
 *
 * Date         Author  Changes
 */
package org.openorb.test.dynamic;

import org.omg.PortableServer.POA;
import org.openorb.test.CORBA.CORBATestCase;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

public class DynAnyTest extends CORBATestCase {    
  public DynAnyTest( String name ) {
    super( name );
  }
    
  public void setUp() {
    super.setUp();

 		try {
      _orb = getORB();

      _any = _orb.create_any();
  			
      org.omg.CORBA.Object obj = _orb.resolve_initial_references("DynAnyFactory");
         
      _dyn_factory = org.omg.DynamicAny.DynAnyFactoryHelper.narrow( obj );    
    }
    catch(org.omg.CORBA.UserException ex) {
      fail("exception during setup:" + ex.toString());
    }
  }
  
  private org.omg.CORBA.ORB _orb;
  private org.omg.CORBA.Any _any;

  private org.omg.DynamicAny.DynAnyFactory _dyn_factory;         
  
  /**
   * Test the various basic inserting within the DynAnys. The DynAny is created, a corresponding value is inserted and
   * then the DynAny is destroyed to avoid memory leak (explicit destruction is necessary).
   * The tested types are char, wchar, boolean, octet, short, ushort, long, ulong, longlong, ulonglong,
   * float, double, any, string, wstring, typecode, object.
   * Basic DynAny operations (rewind, component_count) are also tested.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testDynBasic() throws org.omg.CORBA.UserException {
    org.omg.DynamicAny.DynAny dynAny;

    dynAny = _dyn_factory.create_dyn_any_from_type_code(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_char));
    dynAny.insert_char('c');
    dynAny.get_char();

    dynAny.type();
    dynAny.destroy();

    dynAny = _dyn_factory.create_dyn_any_from_type_code(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_wchar));
    dynAny.insert_wchar('c');
    dynAny.get_wchar();
    dynAny.destroy();

    dynAny = _dyn_factory.create_dyn_any_from_type_code(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_boolean));
    dynAny.insert_boolean(true);
    dynAny.get_boolean();
    dynAny.destroy();

    dynAny = _dyn_factory.create_dyn_any_from_type_code(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_octet));
    dynAny.insert_octet((byte)'o');
    dynAny.get_octet();
    dynAny.destroy();

    dynAny = _dyn_factory.create_dyn_any_from_type_code(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_short));
    dynAny.insert_short((short)1);
    dynAny.get_short();
    dynAny.destroy();

    dynAny = _dyn_factory.create_dyn_any_from_type_code(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_ushort));
    dynAny.insert_ushort((short)1);
    dynAny.get_ushort();
    dynAny.destroy();

    dynAny = _dyn_factory.create_dyn_any_from_type_code(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_long));
    dynAny.insert_long(2);
    dynAny.get_long();
    dynAny.destroy();

    dynAny = _dyn_factory.create_dyn_any_from_type_code(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_ulong));
    dynAny.insert_ulong(2);
    dynAny.get_ulong();
    dynAny.destroy();

    dynAny = _dyn_factory.create_dyn_any_from_type_code(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_longlong));
    dynAny.insert_longlong(2L);
    dynAny.get_longlong();
    dynAny.destroy();

    dynAny = _dyn_factory.create_dyn_any_from_type_code(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_ulonglong));
    dynAny.insert_ulonglong(2L);
    dynAny.get_ulonglong();
    dynAny.destroy();

    dynAny = _dyn_factory.create_dyn_any_from_type_code(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_float));
    dynAny.insert_float((float)3.0);
    dynAny.get_float();
    dynAny.destroy();

    dynAny = _dyn_factory.create_dyn_any_from_type_code(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_double));
    dynAny.insert_double(3.0);
    dynAny.get_double();
    dynAny.destroy();

    dynAny = _dyn_factory.create_dyn_any_from_type_code(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_any));
    dynAny.insert_any(_any);
    dynAny.get_any();
    dynAny.destroy();

    dynAny = _dyn_factory.create_dyn_any_from_type_code(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_string));   
    dynAny.insert_string("str");
    dynAny.get_string();
    dynAny.destroy();

    dynAny = _dyn_factory.create_dyn_any_from_type_code(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_wstring));
    dynAny.insert_wstring("str");
    dynAny.get_wstring();
    dynAny.destroy();

    dynAny = _dyn_factory.create_dyn_any_from_type_code(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_TypeCode));
    dynAny.insert_typecode(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_TypeCode));
    dynAny.get_typecode();
    dynAny.destroy();

    dynAny = _dyn_factory.create_dyn_any_from_type_code(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_objref));
    dynAny.insert_reference(null);
    dynAny.get_reference();

    // For test scoring only
    dynAny.type();

    dynAny.rewind();

    dynAny.seek(0);

    dynAny.component_count();

    dynAny.next();

    dynAny.current_component();

    dynAny.from_any(dynAny.to_any());

    dynAny.assign(dynAny.copy());

    // For test scoring only    
    ((org.omg.DynamicAny.DynAny)dynAny).rewind();

    ((org.omg.DynamicAny.DynAny)dynAny).seek(0);

    ((org.omg.DynamicAny.DynAny)dynAny).component_count();

    ((org.omg.DynamicAny.DynAny)dynAny).next();

    ((org.omg.DynamicAny.DynAny)dynAny).current_component();

    ((org.omg.DynamicAny.DynAny)dynAny).from_any(((org.omg.DynamicAny.DynAny)dynAny).to_any());

    ((org.omg.DynamicAny.DynAny)dynAny).assign(((org.omg.DynamicAny.DynAny)dynAny).copy());

    ((org.omg.DynamicAny.DynAny)dynAny).equal(((org.omg.DynamicAny.DynAny)dynAny).copy());

    dynAny.destroy(); 
  }
  
  /**
   * Test the various operations inserting within the DynEnums. The DynEnum is created with a TypeCode.
   * DynEnum operations (set_as_string, set_as_ulong ) and DynAny operations (rewind, component_count) are also tested.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testDynEnum() throws org.omg.CORBA.UserException {
    org.omg.DynamicAny.DynEnum dyn_enum;

    dyn_enum = (org.omg.DynamicAny.DynEnum)
    _dyn_factory.create_dyn_any_from_type_code(_orb.create_enum_tc( "IDL:Dummy:1.0", "Dummy", new String[] {"RED", "GREEN"}) );

    dyn_enum.set_as_string("RED");

    dyn_enum.get_as_string();

    dyn_enum.set_as_ulong(0);    

    dyn_enum.get_as_ulong();

    // For test scoring only
    dyn_enum.rewind();

    dyn_enum.seek(0);

    dyn_enum.component_count();

    dyn_enum.next();

    dyn_enum.current_component();

    dyn_enum.from_any(dyn_enum.to_any());

    dyn_enum.assign(dyn_enum.copy());

    dyn_enum.destroy();    
  }
  
  /**
   * Test the various operations inserting within the DynStructs. The DynStruct is created with a TypeCode.
   * DynStruct operations (get/set members) and DynAny operations (rewind, component_count) are also tested.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testDynStruct() throws org.omg.CORBA.UserException {
    org.omg.DynamicAny.DynStruct dyn_struct;

    dyn_struct = (org.omg.DynamicAny.DynStruct)
      _dyn_factory.create_dyn_any_from_type_code(_orb.create_struct_tc( "IDL:Dummy:1.0", "Dummy", new org.omg.CORBA.StructMember[0]) );

    dyn_struct.current_member_name();

    dyn_struct.current_member_kind();

    dyn_struct.get_members();    

    dyn_struct.set_members( new org.omg.DynamicAny.NameValuePair[0]);

    dyn_struct.get_members_as_dyn_any();

    dyn_struct.set_members_as_dyn_any( new org.omg.DynamicAny.NameDynAnyPair[0]);

    // For test scoring only
    dyn_struct.rewind();

    dyn_struct.seek(0);

    dyn_struct.component_count();

    dyn_struct.next();

    dyn_struct.current_component();

    dyn_struct.from_any(dyn_struct.to_any());

    dyn_struct.assign(dyn_struct.copy());

    dyn_struct.destroy();    
  }
  
  /**
   * Test the various operations inserting within the DynFixeds. The DynFixed is created with a TypeCode.
   * DynFixed operations (get/set value) and DynAny operations (rewind, component_count) are also tested.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testDynFixed() throws org.omg.CORBA.UserException {
    org.omg.DynamicAny.DynFixed dyn_fixed;

    dyn_fixed = (org.omg.DynamicAny.DynFixed)
    _dyn_factory.create_dyn_any_from_type_code(_orb.create_fixed_tc( (short)5, (short)2 ));

    dyn_fixed.set_value("0123.450d");

    dyn_fixed.get_value();

    // For test scoring only
    dyn_fixed.rewind();

    dyn_fixed.seek(0);

    dyn_fixed.component_count();

    dyn_fixed.next();

    dyn_fixed.current_component();

    dyn_fixed.from_any(dyn_fixed.to_any());

    dyn_fixed.assign(dyn_fixed.copy());

    dyn_fixed.destroy(); 
  }
  
  /**
   * Test the various operations inserting within the DynUnions. The DynUnion is created with a TypeCode and
   * a short TypeCode for the discriminator.
   * DynUnion operations (get/set discriminator, get/set members) and DynAny operations (rewind, component_count) are also tested.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testDynUnion() throws org.omg.CORBA.UserException {
    org.omg.DynamicAny.DynUnion dyn_union;

    dyn_union = (org.omg.DynamicAny.DynUnion)
    _dyn_factory.create_dyn_any_from_type_code( 
      _orb.create_union_tc( "IDL:Dummy:1.0", "ReallyDummy", 
        _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_short), new org.omg.CORBA.UnionMember[0] ) );


    dyn_union.set_discriminator(_dyn_factory.create_dyn_any_from_type_code( 
      _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_short)) );

    try {
      dyn_union.get_discriminator();
    }
    catch(java.lang.Exception ex) {}     
    dyn_union.has_no_active_member();
    try { 
      dyn_union.set_to_no_active_member();
    }
    catch(java.lang.Exception ex) {}     

    dyn_union.discriminator_kind();
    try {      
      dyn_union.set_to_default_member();
    }
    catch(java.lang.Exception ex) {}     

    dyn_union.member();

    dyn_union.member_name();

    dyn_union.member_kind();

    // For test scoring only
    dyn_union.rewind();

    dyn_union.seek(0);

    dyn_union.component_count();

    dyn_union.next();

    dyn_union.current_component();

    try {
      dyn_union.from_any(dyn_union.to_any());
    }
    catch( java.lang.Exception ex ) {} // Checked later

    try {
      dyn_union.assign(dyn_union.copy());
    }
    catch( java.lang.Exception ex ) {} // Checked later

    dyn_union.destroy(); 
  }
  
  
  /**
   * Test the various operations within the DynSequence The DynSequence is created with a TypeCode and
   * a length.
   * DynSequence operations (get/set elements) and DynAny operations (rewind, component_count) are also tested.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testDynSequence() throws org.omg.CORBA.UserException {
    org.omg.DynamicAny.DynSequence dyn_seq;

    dyn_seq = (org.omg.DynamicAny.DynSequence)
    _dyn_factory.create_dyn_any_from_type_code(_orb.create_sequence_tc( 2, _orb.get_primitive_tc( org.omg.CORBA.TCKind.tk_string ) ) );

    dyn_seq.set_length(2);

    dyn_seq.get_length();

    try {	     
      dyn_seq.set_elements( new org.omg.CORBA.Any[0]);
    }      
    catch( java.lang.Exception ex ) {}

    dyn_seq.get_elements();

    try {
      dyn_seq.set_elements_as_dyn_any(new org.omg.DynamicAny.DynAny[0]);
    }      
    catch( java.lang.Exception ex ) {}

    dyn_seq.get_elements_as_dyn_any();

    // For test scoring only
    dyn_seq.rewind();

    dyn_seq.seek(0);

    dyn_seq.component_count();

    dyn_seq.next();

    dyn_seq.current_component();

    dyn_seq.from_any(dyn_seq.to_any());

    dyn_seq.assign(dyn_seq.copy());

    dyn_seq.destroy(); 
  }
  
   
  /**
   * Test the various operations within the DynArray The DynArray is created with a TypeCode and
   * a short TypeCode for the discriminator.
   * DynArray operations (get/set discriminator, get/set members) and DynAny operations (rewind, component_count) are also tested.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testDynArray() throws org.omg.CORBA.UserException {
    org.omg.DynamicAny.DynArray dyn_arr;

    dyn_arr = (org.omg.DynamicAny.DynArray)
    _dyn_factory.create_dyn_any_from_type_code(_orb.create_array_tc( 2, _orb.get_primitive_tc( org.omg.CORBA.TCKind.tk_string ) ) );

    try {	     
      dyn_arr.set_elements( new org.omg.CORBA.Any[0]);
    }      
    catch( java.lang.Exception ex ) {}

    dyn_arr.get_elements();

    try {
      dyn_arr.set_elements_as_dyn_any(new org.omg.DynamicAny.DynAny[0]);
    }      
    catch( java.lang.Exception ex ) {}

    dyn_arr.get_elements_as_dyn_any();

    // For test scoring only
    dyn_arr.rewind();

    dyn_arr.seek(0);

    dyn_arr.component_count();

    dyn_arr.next();

    dyn_arr.current_component();

    dyn_arr.from_any(dyn_arr.to_any());

    dyn_arr.assign(dyn_arr.copy());

    dyn_arr.destroy(); 
  }
  
  static public void main(String args[])
  {
    junit.textui.TestRunner.run(new TestSuite(DynAnyTest.class));
  }
}