/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Intalio Inc.  For written permission, please
 *    contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Intalio Inc. Exolab is a registered trademark of
 *    Intalio Inc.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY INTALIO AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL INTALIO OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2000 (C) Intalio Inc. All Rights Reserved.
 *
 * $Id: BiDirTest.java,v 1.9 2001/08/13 07:58:37 wood Exp $
 *
 * Date         Author  Changes
 */

package org.openorb.test.iiop.bidir;

import org.omg.CORBA.Policy;
import org.omg.CORBA.PolicyManager;
import org.omg.CORBA.SetOverrideType;
import org.omg.PortableServer.*;
import org.openorb.test.ORBTestCase;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

import java.util.Arrays;

/**
 * Tests bidirectional IIOP.
 * <p>
 *
 * @version $Revision: 1.9 $ $Date: 2001/08/13 07:58:37 $ 
 * @author <a href="mailto:wood@intalio.com">Chris Wood &lt;wood@intalio.com&gt;</a>
 * @see     org.exolab.jtf.CWTestCateogry
 * @see     org.exolab.jtf.CWTestCase
 */
public class BiDirTest extends ORBTestCase
{
  public BiDirTest(String name) {
		super(name);
	}
	
	/**
   * Overloaded to allow the persistant test case to work.
   */
	protected void setUp()
	{
		java.util.Properties props = new java.util.Properties();
    props.setProperty("openorb.useStaticThreadGroup", "true");
		props.setProperty("iiop.biDirOnlyServer", "true");
		setUp(props);
    
		props.setProperty("iiop.biDirOnlyServer", "false");
    orb2 = org.omg.CORBA.ORB.init(new String[0], props);

    Thread curr = Thread.currentThread();
    curr.interrupt();
    orb2.run();
	}

  protected void tearDown() {
    super.tearDown();
    
    orb2.shutdown(true);
  }
  
  private org.omg.CORBA.ORB orb2;
  private static boolean [] _callAt;
  
  /**
   * Test bidirectional iiop.
   * 
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
	public void testBiDir() throws org.omg.CORBA.UserException {
    // find the root poa
    org.omg.CORBA.ORB orb = getORB();

    org.omg.CORBA.Any any = orb.create_any();
    any.insert_ushort(org.omg.BiDirPolicy.BOTH.value);

    POA rootPOA = (POA)orb.resolve_initial_references("RootPOA");
    POA rootPOA2 = (POA)orb2.resolve_initial_references("RootPOA");

    Policy [] policies = new Policy[3];
    policies[0] = orb.create_policy(org.omg.BiDirPolicy.BIDIRECTIONAL_POLICY_TYPE.value, any);
    policies[1] = orb.create_policy(org.openorb.policy.FORCE_MARSHAL_POLICY_ID.value, orb.create_any());
    policies[2] = rootPOA.create_implicit_activation_policy(ImplicitActivationPolicyValue.IMPLICIT_ACTIVATION);

    // set client policies, use bidir and force marshalling
    PolicyManager opm = (PolicyManager)orb.resolve_initial_references("ORBPolicyManager");
    opm.set_policy_overrides(policies, SetOverrideType.ADD_OVERRIDE);
    PolicyManager opm2 = (PolicyManager)orb2.resolve_initial_references("ORBPolicyManager");
    opm2.set_policy_overrides(policies, SetOverrideType.ADD_OVERRIDE);

    // setup poa
    POA poa = rootPOA.create_POA("BidirAdapter", rootPOA.the_POAManager(), policies);
    POA poa2 = rootPOA2.create_POA("BidirAdapter", rootPOA2.the_POAManager(), policies);

    // activate root poa
    rootPOA.the_POAManager().activate();
    rootPOA2.the_POAManager().activate();

    Callback ref1 = (new CallbackImpl(poa))._this(orb);
    Callback ref2 = (new CallbackImpl(poa2))._this(orb2);
    Callback ref2at1 = CallbackHelper.narrow(orb.string_to_object(orb2.object_to_string(ref2)));

    // do the callback
    _callAt = new boolean[2];
    ref2at1.call(ref1, 1);

    for(int i = _callAt.length-1; i >= 0; --i)
      assertTrue("Call level did not reach level "+i, _callAt[i]);
    
	}
  
  /**
   * Test bidirectional iiop with fragmentation.
   * 
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
	public void testBiDirFragmented()
    throws org.omg.CORBA.UserException 
  {
    // find the root poa
    org.omg.CORBA.ORB orb = getORB();

    org.omg.CORBA.Any any = orb.create_any();
    any.insert_ushort(org.omg.BiDirPolicy.BOTH.value);

    POA rootPOA = (POA)orb.resolve_initial_references("RootPOA");
    POA rootPOA2 = (POA)orb2.resolve_initial_references("RootPOA");

    Policy [] policies = new Policy[3];
    policies[0] = orb.create_policy(org.omg.BiDirPolicy.BIDIRECTIONAL_POLICY_TYPE.value, any);
    policies[1] = orb.create_policy(org.openorb.policy.FORCE_MARSHAL_POLICY_ID.value, orb.create_any());
    policies[2] = rootPOA.create_implicit_activation_policy(ImplicitActivationPolicyValue.IMPLICIT_ACTIVATION);

    // set client policies, use bidir and force marshalling
    PolicyManager opm = (PolicyManager)orb.resolve_initial_references("ORBPolicyManager");
    opm.set_policy_overrides(policies, SetOverrideType.ADD_OVERRIDE);
    PolicyManager opm2 = (PolicyManager)orb2.resolve_initial_references("ORBPolicyManager");
    opm2.set_policy_overrides(policies, SetOverrideType.ADD_OVERRIDE);

    // setup poa
    POA poa = rootPOA.create_POA("BidirAdapter", rootPOA.the_POAManager(), policies);
    POA poa2 = rootPOA2.create_POA("BidirAdapter", rootPOA2.the_POAManager(), policies);

    // activate root poa
    rootPOA.the_POAManager().activate();
    rootPOA2.the_POAManager().activate();

    Callback ref1 = (new CallbackImpl(poa))._this(orb);
    Callback ref2 = (new CallbackImpl(poa2))._this(orb2);
    Callback ref2at1 = CallbackHelper.narrow(orb.string_to_object(orb2.object_to_string(ref2)));

    // do the callback
    _callAt = new boolean[4];
    ref2at1.call_large(ref1, 3, new byte[1024*1024]);

    for(int i = _callAt.length-1; i >= 0; --i)
      assertTrue("Call level did not reach level "+i, _callAt[i]);
    
	}
  
	private static class CallbackImpl extends CallbackPOA {
		CallbackImpl(POA poa) {
      _poa = poa;
    }

    private POA _poa;
    
    public POA _default_POA() {
      return _poa;
    }
		
		public void call(org.openorb.test.iiop.bidir.Callback call,int remain) 
		{
      _callAt[remain] = true;
      if(remain > 0) 
        call.call(_this(), remain-1);
		}
    
    public void call_large(org.openorb.test.iiop.bidir.Callback call, 
                           int remain, byte[] buffer) 
    {
      _callAt[remain] = true;
      if(remain > 0)
        call.call_large(_this(), remain-1, buffer);
    }
    
	}
  
  public static void main(String[] args)
	{
		junit.textui.TestRunner.run(new TestSuite(BiDirTest.class));
	}  
}