/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Intalio Inc.  For written permission, please
 *    contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Intalio Inc. Exolab is a registered trademark of
 *    Intalio Inc.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY INTALIO AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL INTALIO OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2000 (C) Intalio Inc. All Rights Reserved.
 *
 * $Id: ComplexTest.java,v 1.3 2001/08/13 03:47:34 wood Exp $
 *
 * Date         Author  Changes
 */

package org.openorb.test.iiop.complex;

import org.omg.CORBA.Policy;
import org.omg.PortableServer.*;
import org.openorb.test.ORBTestCase;
import junit.framework.TestSuite;

import java.util.Arrays;

/**
 * Tests marshaling and unmarshaling of various iiop types.
 *
 * @version $Revision: 1.3 $ $Date: 2001/08/13 03:47:34 $ 
 * @author <a href="mailto:wood@intalio.com">Chris Wood &lt;wood@intalio.com&gt;</a>
 * @see     org.exolab.jtf.CWTestCateogry
 * @see     org.exolab.jtf.CWTestCase
 */
public class ComplexTest extends ORBTestCase
{
	public ComplexTest(String name) {
		super(name);
	}
	
	protected void setUp() {
		super.setUp();

		try {
			_orb = getORB();

			POA rootPOA = (POA)_orb.resolve_initial_references("RootPOA");

			EchoComplex svr_ref = (new EchoComplexImpl(rootPOA))._this(_orb);

			rootPOA.the_POAManager().activate();

			_cltRef = EchoComplexHelper.narrow(forceMarshal(svr_ref));

			_any = _orb.create_any();
		}
		catch(org.omg.CORBA.UserException ex) {
			fail("exception during setup:" + ex.toString());
		}
	}
		
	private EchoComplex _cltRef;
	private org.omg.CORBA.ORB _orb;
	private org.omg.CORBA.Any _any;
	private org.omg.CORBA.Any _rpl;

	/**
	 * Simple struct echo.
	 */
	public void testSimpleStruct() {
    SimpleStruct ss = new SimpleStruct(1);
    
    SimpleStruct ret = _cltRef.echo_simple_s(ss);
    
    assertEquals("Body of returned struct not equal", ss.n, ret.n);
    
    SimpleStructHelper.insert(_any, ss);
    
    _rpl = _cltRef.echo_any(_any);
    
    assertTrue("Returned any is not identical to original", _rpl.equal(_any));
  }
  
	/**
	 * Complex struct echo.
	 */
	public void testComplexStruct() {
    ComplexStruct s = new ComplexStruct(new SimpleStruct(1), "a string");
    
    ComplexStruct ret = _cltRef.echo_complex_s(s);
    
    assertEquals("Body of returned struct not equal", s.s, ret.s);
    
    ComplexStructHelper.insert(_any, s);
    
    _rpl = _cltRef.echo_any(_any);
    
    assertTrue("Returned any is not identical to original", _rpl.equal(_any));
  }
  
	/**
	 * Recursive struct echo.
	 */
	public void testRecursiveStruct() {
		RecursiveStruct [] e = new RecursiveStruct[0];
		
		RecursiveStruct [] left = new RecursiveStruct[] { new RecursiveStruct("left", e, e) };
		RecursiveStruct [] right = new RecursiveStruct[] { new RecursiveStruct("right", e, e) };
		
    RecursiveStruct s = new RecursiveStruct("top", left, right);
    
    RecursiveStruct ret = _cltRef.echo_recursive_s(s);    
    
    RecursiveStructHelper.insert(_any, s);
    
    _rpl = _cltRef.echo_any(_any);
    
    assertTrue("Returned any is not identical to original", _rpl.equal(_any));
  }
  
 /**
	 * Fixed type echo.
	 */
	public void testFixed() {
    java.math.BigDecimal f = new java.math.BigDecimal( 10/3 );
    
    java.math.BigDecimal ret = _cltRef.echo_fixed(f);
    
    assertEquals("Body of returned fixed not equal", f, ret);
    
    Fixed52Helper.insert(_any, f);
    
    _rpl = _cltRef.echo_any(_any);
    
    assertTrue("Returned any is not identical to original", _rpl.equal(_any));
  }

  
  static class EchoComplexImpl extends EchoComplexPOA {
    EchoComplexImpl(POA poa) {
      _poa = poa;
    }

    private POA _poa;
    
    public POA _default_POA() {
      return _poa;
    }

    public org.omg.CORBA.Any echo_any(org.omg.CORBA.Any a) {
      return a;
    }
        
    public org.openorb.test.iiop.complex.SimpleStruct echo_simple_s(org.openorb.test.iiop.complex.SimpleStruct ss) {
      return ss;
    }
    
    public org.openorb.test.iiop.complex.ComplexStruct echo_complex_s(org.openorb.test.iiop.complex.ComplexStruct ss) {
      return ss;
    }
    
    public org.openorb.test.iiop.complex.RecursiveStruct echo_recursive_s(org.openorb.test.iiop.complex.RecursiveStruct rs) {
      return rs;
    }
    
    public org.openorb.test.iiop.complex.SimpleEnum echo_simple_e(org.openorb.test.iiop.complex.SimpleEnum se) {
      return se;
    }

    public org.openorb.test.iiop.complex.EnumUnion echo_enum_u(org.openorb.test.iiop.complex.EnumUnion eu) {
      return eu;
    }    
    
    public org.openorb.test.iiop.complex.LongUnion echo_long_u(org.openorb.test.iiop.complex.LongUnion lu) {
      return lu;
    }
    
    
    public java.math.BigDecimal echo_fixed(java.math.BigDecimal fi) {
      return fi;
    }
  }
  
  static public void main(String args[])
	{
		junit.textui.TestRunner.run(new TestSuite(ComplexTest.class));
	}
}