/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Intalio Inc.  For written permission, please
 *    contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Intalio Inc. Exolab is a registered trademark of
 *    Intalio Inc.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY INTALIO AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL INTALIO OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2000 (C) Intalio Inc. All Rights Reserved.
 *
 * $Id: PrimativeTest.java,v 1.9 2001/08/13 03:47:34 wood Exp $
 *
 * Date         Author  Changes
 */

package org.openorb.test.iiop.primative;

import org.omg.CORBA.Policy;
import org.omg.PortableServer.*;
import org.openorb.test.ORBTestCase;
import junit.framework.TestSuite;

import java.util.Arrays;
import java.util.Properties;

/**
 * Tests marshaling and unmarshaling of various iiop types.
 *
 * @version $Revision: 1.9 $ $Date: 2001/08/13 03:47:34 $ 
 * @author <a href="mailto:wood@intalio.com">Chris Wood &lt;wood@intalio.com&gt;</a>
 * @see     org.exolab.jtf.CWTestCateogry
 * @see     org.exolab.jtf.CWTestCase
 */
public class PrimativeTest extends ORBTestCase
{
	public PrimativeTest(String name) {
		super(name);
	}
	
	protected void setUp() {
		super.setUp();

		try {
			_orb = getORB();

			POA rootPOA = (POA)_orb.resolve_initial_references("RootPOA");

			_svrRef = (new EchoImpl(rootPOA))._this(_orb);

			rootPOA.the_POAManager().activate();

			_cltRef = EchoHelper.narrow(forceMarshal(_svrRef));

			_any = _orb.create_any();
		}
		catch(org.omg.CORBA.UserException ex) {
			fail("exception during setup:" + ex.toString());
		}
	}
		
	private Echo _svrRef;
	private Echo _cltRef;
	private org.omg.CORBA.ORB _orb;
	private org.omg.CORBA.Any _any;
	private org.omg.CORBA.Any _rpl;

	/**
	 * Test basic object operations.
	 */
	public void testBasicOperations() {
    assertTrue("Object is local", !((org.omg.CORBA.portable.ObjectImpl)_cltRef)._is_local());
    
		assertTrue("Object does not exist", !_cltRef._non_existent());
		
		assertTrue("Object is not a \"IDL:omg.org/CORBA/Object:1.0\"",
			_cltRef._is_a("IDL:omg.org/CORBA/Object:1.0"));

    String nomin = EchoHelper.id();
    nomin = nomin.substring(0, nomin.lastIndexOf('.'));
    String overs = nomin + ".0";
    String nvers = nomin + ".2";
    
    // client side check
		assertTrue("Object is not a \""+EchoHelper.id()+"\"",
			_cltRef._is_a(EchoHelper.id()));
		    
		assertTrue("Object is not a \""+overs+"\"",
			_cltRef._is_a(overs));
    
		assertTrue("Object is a \""+nvers+"\"",
			!_cltRef._is_a(nvers));
    
		assertTrue("Object is a \"IDL:com.beezwax/Bumblebee:1.0\"",
			!_cltRef._is_a("IDL:com.beezwax/Bumblebee:1.0"));
		    
		org.omg.CORBA.DomainManager [] managers = _cltRef._get_domain_managers();
          
		try {
			_cltRef._get_interface_def();
		}
		catch(org.omg.CORBA.INTF_REPOS ex) {
		}
    
    _cltRef.toString();
  }

  /**
   * Test local invocation.
   */
  public void testLocalInvocation() {
    assertTrue("Object is not local", ((org.omg.CORBA.portable.ObjectImpl)_svrRef)._is_local());
    
		assertTrue("Object does not exist", !_svrRef._non_existent());
		
		assertTrue("Object is not a \"IDL:omg.org/CORBA/Object:1.0\"",
			_svrRef._is_a("IDL:omg.org/CORBA/Object:1.0"));

		assertTrue("Object is not a \""+EchoHelper.id()+"\"",
			_svrRef._is_a(EchoHelper.id()));
		
		assertTrue("Object is a \"IDL:com.beezwax/Bumblebee:1.0\"",
			!_svrRef._is_a("IDL:com.beezwax/Bumblebee:1.0"));
		
		org.omg.CORBA.DomainManager [] managers = _svrRef._get_domain_managers();
          
		try {
			_svrRef._get_interface_def();
		}
		catch(org.omg.CORBA.INTF_REPOS ex) {
		}
 
    _svrRef.ping();
  }
  
  /**
   * Test interorb invocation.
   */
  public void testInterORBInvocation()
    throws Exception 
  {
    Properties props = new Properties();
    props.setProperty("openorb.useStaticThreadGroup", "true");
    org.omg.CORBA.ORB nlORB = org.omg.CORBA.ORB.init(new String[0], props);
    
    org.omg.CORBA.Object nlObj = nlORB.string_to_object(_orb.object_to_string(_svrRef));
    
    assertTrue("Object is local", !((org.omg.CORBA.portable.ObjectImpl)nlObj)._is_local());
    
		assertTrue("Object does not exist", !nlObj._non_existent());
    
    EchoHelper.narrow(_svrRef).ping();
    
    nlORB.shutdown(true);
  }
  
	/**
	 * Test object ref transmission.
	 */
	public void testObject() {
		assertTrue("Object is not equivalent to self",
			_cltRef.echo_Object(_cltRef)._is_equivalent(_cltRef));
				
		_any.insert_Object(_cltRef);
		_rpl = _cltRef.echo_any(_any);

		assertTrue("failed object reference any contained",
			_any.equal(_rpl));

		assertTrue("failed object reference any",
			_rpl.extract_Object()._is_equivalent(_cltRef));
	}					
	
	/**
	 * Test ping/void
	 */
	public void testVoid() {
		_cltRef.ping();

		_any.type(_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_void));
		if(_cltRef.echo_any(_any).type().kind() != org.omg.CORBA.TCKind.tk_void)
			fail("failed any void");

		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed typecode void");
	}
	
	/**
	 * Throw exception
	 */
	public void testThrowException() {
		// throw exception
		try {
			_cltRef.raise_exception();
			fail("exception not raised");
		}
		catch(org.omg.CORBA.UNKNOWN ex) {
			// expected.
		}
	}

	/**
	 * Test boolean echo.
	 */
	public void testBoolean() {
		final boolean [] vals = { true, false };

		for(int i = 0; i < vals.length; ++i) {
			if(_cltRef.echo_boolean(vals[i]) != vals[i])
				fail("failed boolean " + (vals[i]?"true":"false"));

			_any.insert_boolean(vals[i]);
			_rpl = _cltRef.echo_any(_any);
			if(!_any.equal(_rpl))
				fail("Equality test failed for boolean any " + (vals[i]?"true":"false"));

			if(_rpl.extract_boolean() != vals[i])
				fail("failed boolean any " + (vals[i]?"true":"false"));
		}

		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed boolean typecode");


		if(!Arrays.equals(vals, _cltRef.echo_boolean_s(vals)))
			fail("Sequence test failed for boolean");

		org.omg.CORBA.BooleanSeqHelper.insert(_any, vals);
		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed boolean seq typecode");

		_rpl = _cltRef.echo_any(_any);
		if(!_any.equal(_rpl))
			fail("Equality test failed for boolean any sequence");
	}

	/**
	 * Test octet echo.
	 */
	public void testOctet() {
		final byte [] vals = { (byte)0, (byte)1, (byte)7, (byte)0x1F, (byte)0x80, (byte)0xFF };

		for(int i = 0; i < vals.length; ++i) {
			if(_cltRef.echo_octet(vals[i]) != vals[i])
				fail("failed octet " + vals[i]);

			_any.insert_octet(vals[i]);
			_rpl = _cltRef.echo_any(_any);
			if(!_any.equal(_rpl))
				fail("Equality test failed for octet any " + vals[i]);

			if(_rpl.extract_octet() != vals[i])
				fail("failed octet any " + vals[i]);
		}

		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed octet typecode");

		if(!Arrays.equals(vals, _cltRef.echo_octet_s(vals)))
			fail("Sequence test failed for octet");

		org.omg.CORBA.OctetSeqHelper.insert(_any, vals);
		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed octet seq typecode");

		_rpl = _cltRef.echo_any(_any);
		if(!_any.equal(_rpl))
			fail("Equality test failed for boolean any sequence");
	}

	/**
	 * Test short echo.
	 */
	public void testShort() {
		final short [] vals = { (short)0, (short)1, (short)0xFF, (short)0x2345, (short)0x6767, (short)0x8077, (short)0xFFFF };

		for(int i = 0; i < vals.length; ++i) {
			if(_cltRef.echo_short(vals[i]) != vals[i])
				fail("failed short " + vals[i]);

			_any.insert_short(vals[i]);
			_rpl = _cltRef.echo_any(_any);
			if(!_any.equal(_rpl))
				fail("Equality test failed for short any " + vals[i]);

			if(_rpl.extract_short() != vals[i])
				fail("failed short any " + vals[i]);
		}

		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed short typecode");

		if(!Arrays.equals(vals, _cltRef.echo_short_s(vals)))
			fail("Sequence test failed for short");

		org.omg.CORBA.ShortSeqHelper.insert(_any, vals);
		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed short seq typecode");

		_rpl = _cltRef.echo_any(_any);
		if(!_any.equal(_rpl))
			fail("Equality test failed for short any sequence");
	}

	/**
	 * Test unsigned short echo.
	 */
	public void testUShort() {
		final short [] vals = { (short)0, (short)1, (short)0xFF, (short)0x2345, (short)0x6767, (short)0x8077, (short)0xFFFF };

		for(int i = 0; i < vals.length; ++i) {
			if(_cltRef.echo_ushort(vals[i]) != vals[i])
				fail("failed ushort " + vals[i]);

			_any.insert_ushort(vals[i]);
			_rpl = _cltRef.echo_any(_any);
			if(!_any.equal(_rpl))
				fail("Equality test failed for ushort any " + vals[i]);

			if(_rpl.extract_ushort() != vals[i])
				fail("failed ushort any " + vals[i]);
		}

		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed ushort typecode");

		if(!Arrays.equals(vals, _cltRef.echo_ushort_s(vals)))
			fail("Sequence test failed for ushort");

		org.omg.CORBA.UShortSeqHelper.insert(_any, vals);
		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed ushort seq typecode");

		_rpl = _cltRef.echo_any(_any);
		if(!_any.equal(_rpl))
			fail("Equality test failed for ushort any sequence");
	}

	/**
	 * Test long echo.
	 */
	public void testLong() {
		int [] vals = { 0, 1, 0xFF, 0xFFFF, 0xFFF679, 0x80079874, 0xFFFFFFFF, 0x7FFFFFFF };

		for(int i = 0; i < vals.length; ++i) {
			if(_cltRef.echo_long(vals[i]) != vals[i])
				fail("failed long " + vals[i]);

			_any.insert_long(vals[i]);
			_rpl = _cltRef.echo_any(_any);
			if(!_any.equal(_rpl))
				fail("Equality test failed for long any " + vals[i]);

			if(_rpl.extract_long() != vals[i])
				fail("failed long any " + vals[i]);
		}

		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed long typecode");

		if(!Arrays.equals(vals, _cltRef.echo_long_s(vals)))
			fail("Sequence test failed for long");

		org.omg.CORBA.LongSeqHelper.insert(_any, vals);
		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed long seq typecode");

		_rpl = _cltRef.echo_any(_any);
		if(!_any.equal(_rpl))
			fail("Equality test failed for long any sequence");
	}

	/**
	 * Test unsigned long echo.
	 */
	public void testULong() {
		int [] vals = { 0, 1, 0xFF, 0xFFFF, 0xFFF679, 0x80079874, 0xFFFFFFFF, 0x7FFFFFFF };

		for(int i = 0; i < vals.length; ++i) {
			if(_cltRef.echo_ulong(vals[i]) != vals[i])
				fail("failed ulong " + vals[i]);

			_any.insert_ulong(vals[i]);
			_rpl = _cltRef.echo_any(_any);
			if(!_any.equal(_rpl))
				fail("Equality test failed for ulong any " + vals[i]);

			if(_rpl.extract_ulong() != vals[i])
				fail("failed ulong any " + vals[i]);
		}

		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed ulong typecode");

		if(!Arrays.equals(vals, _cltRef.echo_ulong_s(vals)))
			fail("Sequence test failed for ulong");

		org.omg.CORBA.ULongSeqHelper.insert(_any, vals);
		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed ulong seq typecode");

		_rpl = _cltRef.echo_any(_any);
		if(!_any.equal(_rpl))
			fail("Equality test failed for ulong any sequence");
	}

	/**
	 * Test long long echo.
	 */
	public void testLongLong() {
		long [] vals = { 0l, 1l, 0xFFl, 0xFFFFl, 0xFFF679l, 0x80079874l, 
			0xFFFFFFFFl, 0x7FFFFFFFl, 0xFF320984723l, 0x709808576FFFACFBl,
			0xFFFFFFFFFFFFFFFFl, 0x7FFFFFFFFFFFFFFFl };

		for(int i = 0; i < vals.length; ++i) {
			if(_cltRef.echo_longlong(vals[i]) != vals[i])
				fail("failed long long " + vals[i]);

			_any.insert_longlong(vals[i]);
			_rpl = _cltRef.echo_any(_any);
			if(!_any.equal(_rpl))
				fail("Equality test failed for long long any " + vals[i]);

			if(_rpl.extract_longlong() != vals[i])
				fail("failed long long any " + vals[i]);
		}

		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed long long typecode");

		if(!Arrays.equals(vals, _cltRef.echo_longlong_s(vals)))
			fail("Sequence test failed for long long");

		org.omg.CORBA.LongLongSeqHelper.insert(_any, vals);
		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed long long seq typecode");

		_rpl = _cltRef.echo_any(_any);
		if(!_any.equal(_rpl))
			fail("Equality test failed for long long any sequence");
	}

	/**
	 * Test unsigned long long echo.
	 */
	public void testULongLong() {
		long [] vals = { 0l, 1l, 0xFFl, 0xFFFFl, 0xFFF679l, 0x80079874l, 
			0xFFFFFFFFl, 0x7FFFFFFFl, 0xFF320984723l, 0x709808576FFFACFBl,
			0xFFFFFFFFFFFFFFFFl, 0x7FFFFFFFFFFFFFFFl };

		for(int i = 0; i < vals.length; ++i) {
			if(_cltRef.echo_ulonglong(vals[i]) != vals[i])
				fail("failed ulonglong " + vals[i]);

			_any.insert_ulonglong(vals[i]);
			_rpl = _cltRef.echo_any(_any);
			if(!_any.equal(_rpl))
				fail("Equality test failed for ulonglong any " + vals[i]);

			if(_rpl.extract_ulonglong() != vals[i])
				fail("failed ulonglong any " + vals[i]);
		}

		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed ulonglong typecode");

		if(!Arrays.equals(vals, _cltRef.echo_ulonglong_s(vals)))
			fail("Sequence test failed for ulonglong");

		org.omg.CORBA.ULongLongSeqHelper.insert(_any, vals);
		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed ulonglong seq typecode");

		_rpl = _cltRef.echo_any(_any);
		if(!_any.equal(_rpl))
			fail("Equality test failed for ulonglong any sequence");
	}

	/**
	 * Test float echo.
	 */
	public void testFloat() {
		final float [] vals = { (float)0.0, (float)-0.0, Float.MAX_VALUE, Float.MIN_VALUE, Float.NEGATIVE_INFINITY, Float.POSITIVE_INFINITY };

		for(int i = 0; i < vals.length; ++i) {
			if(_cltRef.echo_float(vals[i]) != vals[i])
				fail("failed float " + vals[i]);

			_any.insert_float(vals[i]);
			_rpl = _cltRef.echo_any(_any);
			if(!_any.equal(_rpl))
				fail("Equality test failed for float any " + vals[i]);

			if(_rpl.extract_float() != vals[i])
				fail("failed float any " + vals[i]);
		}

		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed float typecode");

		if(!Arrays.equals(vals, _cltRef.echo_float_s(vals)))
			fail("Sequence test failed for float");

		org.omg.CORBA.FloatSeqHelper.insert(_any, vals);
		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed float seq typecode");

		_rpl = _cltRef.echo_any(_any);
		if(!_any.equal(_rpl))
			fail("Equality test failed for float any sequence");

		// extra tests for NaNs
		if(!Float.isNaN(_cltRef.echo_float(Float.NaN)))
			fail("failed float " + Float.NaN);

		_any.insert_float(Float.NaN);
		_rpl = _cltRef.echo_any(_any);
		if(!_any.equal(_rpl))
			fail("Equality test failed for float any " + Float.NaN);

		if(!Float.isNaN(_rpl.extract_float()))
			fail("failed float any " + Float.NaN);
	}

	/**
	 * Test double echo.
	 */
	public void testDoubleEcho() {
		final double [] vals = { 0.0, -0.0, Double.MAX_VALUE, Double.MIN_VALUE, Double.NEGATIVE_INFINITY, Double.POSITIVE_INFINITY };

		for(int i = 0; i < vals.length; ++i) {
			if(_cltRef.echo_double(vals[i]) != vals[i])
				fail("failed double " + vals[i]);

			_any.insert_double(vals[i]);
			_rpl = _cltRef.echo_any(_any);
			if(!_any.equal(_rpl))
				fail("Equality test failed for double any " + vals[i]);

			if(_rpl.extract_double() != vals[i])
				fail("failed double any " + vals[i]);
		}

		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed double typecode");

		if(!Arrays.equals(vals, _cltRef.echo_double_s(vals)))
			fail("Sequence test failed for double");

		org.omg.CORBA.DoubleSeqHelper.insert(_any, vals);
		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed double seq typecode");

		_rpl = _cltRef.echo_any(_any);
		if(!_any.equal(_rpl))
			fail("Equality test failed for double any sequence");

		// extra tests for NaNs
		if(!Double.isNaN(_cltRef.echo_double(Double.NaN)))
			fail("failed double " + Double.NaN);

		_any.insert_double(Double.NaN);
		_rpl = _cltRef.echo_any(_any);
		if(!_any.equal(_rpl))
			fail("Equality test failed for double any " + Double.NaN);

		if(!Double.isNaN(_rpl.extract_double()))
			fail("failed double any " + Double.NaN);
	}

	/**
	 * Test char echo.
	 */
	public void testChar() {
		final char [] testvals = { '\u0001', '\u0020', '\u00FF' };
    char [] vals = testvals;
    
		for(int i = 0; i < vals.length; ++i) {
      try {
        if(_cltRef.echo_char(vals[i]) != vals[i])
          fail("failed char " + vals[i]);
        
        _any.insert_char(vals[i]);
        _rpl = _cltRef.echo_any(_any);
        if(!_any.equal(_rpl))
          fail("Equality test failed for char any " + vals[i]);

        if(_rpl.extract_char() != vals[i])
          fail("failed char any " + vals[i]);
      }
      catch(org.omg.CORBA.DATA_CONVERSION ex) {
        // this can occour if the char is a representation of a multibyte char
        char [] old = vals;
        vals = new char[vals.length-1];
        System.arraycopy(old, 0, vals, 0, i);
        if(i+1 < vals.length)
          System.arraycopy(old, i+1, vals, i, vals.length-1);
        --i;
      }
		}

		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed char typecode");

		if(!Arrays.equals(vals, _cltRef.echo_char_s(vals)))
			fail("Sequence test failed for char");

		org.omg.CORBA.CharSeqHelper.insert(_any, vals);
		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed char seq typecode");

		_rpl = _cltRef.echo_any(_any);
		if(!_any.equal(_rpl))
			fail("Equality test failed for char any sequence");
	}

	/**
	 * Test wchar echo.
	 */
	public void testWChar() {
		final char [] vals = { '\u0001', '\u0020', '\u008D', '\u00FF', '\u0626', '\u7554', '\uF7F5', '\uFFFF' };

		for(int i = 0; i < vals.length; ++i) {
			if(_cltRef.echo_wchar(vals[i]) != vals[i])
				fail("failed wchar " + vals[i]);

			_any.insert_wchar(vals[i]);
			_rpl = _cltRef.echo_any(_any);
			if(!_any.equal(_rpl))
				fail("Equality test failed for wchar any " + vals[i]);

			if(_rpl.extract_wchar() != vals[i])
				fail("failed wchar any " + vals[i]);
		}

		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed wchar typecode");

		if(!Arrays.equals(vals, _cltRef.echo_wchar_s(vals)))
			fail("Sequence test failed for wchar");

		org.omg.CORBA.WCharSeqHelper.insert(_any, vals);
		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed wchar seq typecode");

		_rpl = _cltRef.echo_any(_any);
		if(!_any.equal(_rpl))
			fail("Equality test failed for wchar any sequence");
	}

	/**
	 * Test string echo.
	 */
	public void testStringEcho() {      
		final String [] vals = { "", "Mr Jock, T.V. quiz PhD, bags few lynx.", "\u0001\u0010\u007F\u006D\u00FF" };

		for(int i = 0; i < vals.length; ++i) {
			if(!_cltRef.echo_string(vals[i]).equals(vals[i]))
				fail("failed string " + vals[i]);

			_any.insert_string(vals[i]);
			_rpl = _cltRef.echo_any(_any);
			if(!_any.equal(_rpl))
				fail("Equality test failed for string any " + vals[i]);

			if(!_rpl.extract_string().equals(vals[i]))
				fail("failed string any " + vals[i]);
		}

		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed string typecode");

		if(!Arrays.equals(vals, _cltRef.echo_string_s(vals)))
			fail("Sequence test failed for string");

		org.omg.CORBA.StringSeqHelper.insert(_any, vals);
		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed string seq typecode");

		_rpl = _cltRef.echo_any(_any);
		if(!_any.equal(_rpl))
			fail("Equality test failed for string any sequence");

		// echo limited length string typecode
		org.omg.CORBA.TypeCode tc = _orb.create_string_tc(10);
		if(!_cltRef.echo_TypeCode(tc).equal(tc))
			fail("failed string<10> typecode");
	}

	/**
	 * Test wstring echo.
	 */
	public void testWStringEcho() {
		final String [] vals = { "", "Mr Jock, T.V. quiz PhD, bags few lynx.", "\u0001\u0010\u007F\u008D\u00FF\u05DD\u7FDE\u8D40\uFFFF" };

		for(int i = 0; i < vals.length; ++i) {
			if(!_cltRef.echo_wstring(vals[i]).equals(vals[i]))
				fail("failed wstring " + vals[i]);

			_any.insert_wstring(vals[i]);
			_rpl = _cltRef.echo_any(_any);
			if(!_any.equal(_rpl))
				fail("Equality test failed for wstring any " + vals[i]);

			if(!_rpl.extract_wstring().equals(vals[i]))
				fail("failed wstring any " + vals[i]);
		}

		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed wstring typecode");

		if(!Arrays.equals(vals, _cltRef.echo_wstring_s(vals)))
			fail("Sequence test failed for wstring");

		org.omg.CORBA.WStringSeqHelper.insert(_any, vals);
		if(!_cltRef.echo_TypeCode(_any.type()).equal(_any.type()))
			fail("failed wstring seq typecode");

		_rpl = _cltRef.echo_any(_any);
		if(!_any.equal(_rpl))
			fail("Equality test failed for wstring any sequence");

		// echo limited length wstring typecode
		org.omg.CORBA.TypeCode tc = _orb.create_wstring_tc(10);
		if(!_cltRef.echo_TypeCode(tc).equal(tc))
			fail("failed wstring<10> typecode");
	}

	/**
	 * Test large buffer echo.
	 */
	public void testLargeBufferEcho() {
		byte [] largebuf = new byte[1024*1024];

		(new java.util.Random()).nextBytes(largebuf);

		byte [] resp = _cltRef.echo_octet_s(largebuf);

		if(!Arrays.equals(largebuf, resp))
			fail("failed echo large buffer");
	}
  
  /**
   * Test large array of strings echo.
   */
  public void testLargeStringSeqEcho() {
    String[] strings = new String[ 20000 ];
		for( int i=0; i< strings.length; i++ )
		    strings[i] = new String("" + i );
    
    String [] ret = _cltRef.echo_string_s(strings);
    
		if(!Arrays.equals(strings, ret))
			fail("failed echo large buffer");
  }
  
  static class EchoImpl extends EchoPOA {
    EchoImpl(POA poa) {
      _poa = poa;
    }

    private POA _poa;
    
    public POA _default_POA() {
      return _poa;
    }

    public void ping() {
    }

		public void raise_exception() {
			throw new org.omg.CORBA.UNKNOWN();
		}
		
    public boolean echo_boolean(boolean arg) {
      return arg;
    }

    public boolean[] echo_boolean_s(boolean[] arg) {
      return arg;
    }

    public byte echo_octet(byte arg) {
      return arg;
    }

    public byte[] echo_octet_s(byte[] arg) {
      return arg;
    }

    public short echo_short(short arg) {
      return arg;
    }

    public short[] echo_short_s(short[] arg) {
      return arg;
    }

    public short echo_ushort(short arg) {
			return arg;
    }

    public short[] echo_ushort_s(short[] arg) {
      return arg;
    }

    public int echo_long(int arg) {
      return arg;
    }

    public int[] echo_long_s(int[] arg) {
      return arg;
    }

    public int echo_ulong(int arg) {
      return arg;
    }

    public int[] echo_ulong_s(int[] arg) {
      return arg;
    }

    public long echo_longlong(long arg) {
      return arg;
    }

    public long[] echo_longlong_s(long[] arg) {
      return arg;
    }

    public long echo_ulonglong(long arg) {
      return arg;
    }

    public long[] echo_ulonglong_s(long[] arg) {
      return arg;
    }

    public float echo_float(float arg) {
      return arg;
    }

    public float[] echo_float_s(float[] arg) {
      return arg;
    }

    public double echo_double(double arg) {
      return arg;
    }

    public double[] echo_double_s(double[] arg) {
      return arg;
    }

    public char echo_char(char arg) {
      return arg;
    }

    public char[] echo_char_s(char[] arg) {
      return arg;
    }

    public char echo_wchar(char arg) {
      return arg;
    }

    public char[] echo_wchar_s(char[] arg) {
      return arg;
    }

    public java.lang.String echo_string(java.lang.String arg) {
      return arg;
    }

    public java.lang.String[] echo_string_s(java.lang.String[] arg) {
      return arg;
    }

    public java.lang.String echo_wstring(java.lang.String arg) {
      return arg;
    }

    public java.lang.String[] echo_wstring_s(java.lang.String[] arg) {
      return arg;
    }

    public org.omg.CORBA.Object echo_Object(org.omg.CORBA.Object arg) {
      return arg;
    }

    public org.omg.CORBA.Any echo_any(org.omg.CORBA.Any arg) {
      return arg;
    }

    public org.omg.CORBA.Any[] echo_any_s(org.omg.CORBA.Any[] arg) {
      return arg;
    }

    public org.omg.CORBA.TypeCode echo_TypeCode(org.omg.CORBA.TypeCode arg) {
      return arg;
    }
  }
  
  static public void main(String args[])
	{
		junit.textui.TestRunner.run(new TestSuite(PrimativeTest.class));
	}
}