/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Intalio Inc.  For written permission, please
 *    contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Intalio Inc. Exolab is a registered trademark of
 *    Intalio Inc.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY INTALIO AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL INTALIO OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2000 (C) Intalio Inc. All Rights Reserved.
 *
 * $Id: StateTest.java,v 1.5 2001/07/04 09:40:39 wood Exp $
 *
 * Date         Author  Changes
 */

package org.openorb.test.iiop.state;

import org.omg.CORBA.Policy;
import org.omg.CORBA.PolicyManager;
import org.omg.CORBA.SetOverrideType;
import org.omg.PortableServer.*;
import org.openorb.test.ORBTestCase;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

import java.util.Arrays;
import java.util.Properties;

/**
 * Tests bidirectional IIOP.
 * <p>
 *
 * @version $Revision: 1.5 $ $Date: 2001/07/04 09:40:39 $ 
 * @author <a href="mailto:wood@intalio.com">Chris Wood &lt;wood@intalio.com&gt;</a>
 * @see     org.exolab.jtf.CWTestCateogry
 * @see     org.exolab.jtf.CWTestCase
 */
public class StateTest extends TestCase
{
  public StateTest(String name) {
		super(name);
	}
  
  private static final int REPEATS = 5;
  
  private Policy _biDirPolicy = null;
  private org.omg.CORBA.ORB _orb;
  private org.omg.CORBA.ORB _orb2;
  
  private StateTarget _ref;
  private StateTarget _ref2;
	
  /**
   * This is the specialized setup, called directly from the test functions
   */
  private void setUp(Properties props, boolean withBiDir) throws org.omg.CORBA.UserException {
    if(props == null)
      props = new Properties();
    props.setProperty("openorb.useStaticThreadGroup", "true");
    
    if(withBiDir) {
      props.setProperty("iiop.allowBidir", "true");
      _orb2 = org.omg.CORBA.ORB.init((String [])null, props);
      
      org.omg.CORBA.Any any = _orb2.create_any();
      any.insert_ushort(org.omg.BiDirPolicy.BOTH.value);
      _biDirPolicy = _orb2.create_policy(org.omg.BiDirPolicy.BIDIRECTIONAL_POLICY_TYPE.value, any);
      
      Thread.currentThread().interrupt();
      _orb2.run();
      Thread.currentThread().interrupted();
    }
    
    _orb = org.omg.CORBA.ORB.init((String []) null, props);
    
    Thread.currentThread().interrupt();
    _orb.run();
    Thread.currentThread().interrupted();
        
    _ref = initRef(_orb);

    if(withBiDir)
      _ref2 = StateTargetHelper.narrow(_orb.string_to_object(_orb2.object_to_string(initRef(_orb2))));
  }
  
  public void tearDown() {
    if(_orb != null)
      _orb.shutdown(true);
    if(_orb2 != null)
      _orb2.shutdown(true);
  }
  
  private StateTarget initRef(org.omg.CORBA.ORB orb) throws org.omg.CORBA.UserException {
    POA rootPOA = (POA)orb.resolve_initial_references("RootPOA");

    Policy [] policies = new Policy[(_biDirPolicy == null)?2:3];
    policies[0] = orb.create_policy(org.openorb.policy.FORCE_MARSHAL_POLICY_ID.value, orb.create_any());
    policies[1] = rootPOA.create_implicit_activation_policy(ImplicitActivationPolicyValue.IMPLICIT_ACTIVATION);
    if(_biDirPolicy != null)
      policies[2] = _biDirPolicy;

    // set client policies, use bidir and force marshalling
    PolicyManager opm = (PolicyManager)orb.resolve_initial_references("ORBPolicyManager");
    opm.set_policy_overrides(policies, SetOverrideType.ADD_OVERRIDE);

    // setup poa
    POA poa = rootPOA.create_POA("BidirAdapter", rootPOA.the_POAManager(), policies);

    // activate root poa
    rootPOA.the_POAManager().activate();

    return (new StateTargetImpl(poa))._this(orb);
	}

  /**
   * Tests pause from the client side. This test will actualy work only 
   * statisticaly, hence the repeating.
   * 
   * @exception org.omg.CORBA.UserException if any of the test case fails
   * @exception InterruptedException if the thread is interrupted for any reason
   */
  public void testClientPause() throws org.omg.CORBA.UserException, InterruptedException {
    Properties props = new Properties();
    props.setProperty("openorb.client.reapPauseDelay", "200");
    
    setUp(props, false);
    
    for(int i = 0; i < REPEATS; ++i) {
      _ref.ping();
      
      Thread.sleep(500);
    }
  }

  /**
   * Tests pause from the server side. This test will actualy work only 
   * statisticaly, hence the repeating.
   * 
   * @exception org.omg.CORBA.UserException if any of the test case fails
   * @exception InterruptedException if the thread is interrupted for any reason
   */
  public void testServerPause() throws org.omg.CORBA.UserException, InterruptedException {
    Properties props = new Properties();
    props.setProperty("openorb.server.reapCloseDelay", "500");
    
    setUp(props, false);
    
    for(int i = 0; i < REPEATS; ++i) {
      _ref.ping();
      
      Thread.sleep(700);
    }
  }
  
  /**
   * Tests pause from the initiating side of a bidir connection. This test 
   * will actualy work only statisticaly, hence the repeating.
   * 
   * @exception org.omg.CORBA.UserException if any of the test case fails
   * @exception InterruptedException if the thread is interrupted for any reason
   */
  public void testBiDirClientPause() throws org.omg.CORBA.UserException, InterruptedException {
    Properties props = new Properties();
    props.setProperty("openorb.client.reapPauseDelay", "200");
    
    setUp(props, true);
    
    for(int i = 0; i < REPEATS; ++i) {
      _ref2.call_ping(_ref);
      
      Thread.sleep(500);
    }
    
    // try to reuse previous ref in new call
    _ref2.call_ping(null);
  }
  
  /**
   * Tests pause from the listening side of a bidir connection. This test will
   * actualy work only statisticaly, hence the repeating.
   * 
   * @exception org.omg.CORBA.UserException if any of the test case fails
   * @exception InterruptedException if the thread is interrupted for any reason
   */
  public void testBiDirServerPause() throws org.omg.CORBA.UserException, InterruptedException {
    Properties props = new Properties();
    props.setProperty("openorb.server.reapCloseDelay", "700");
    
    setUp(props, true);
    
    for(int i = 0; i < REPEATS; ++i) {
      _ref2.call_ping(_ref);
      
      Thread.sleep(1000);
    }
    
    // try to reuse previous ref in new call
    _ref2.call_ping(null);
  }

  // TODO: add some tests for disorderly shutdown.
  
	private static class StateTargetImpl extends StateTargetPOA {
		StateTargetImpl(POA poa) {
      _poa = poa;
    }

    private POA _poa;
    
    public POA _default_POA() {
      return _poa;
    }
    
    public void ping() {
      int i = 1;
    }
    
    public boolean sleep(long time) {
      try {
        Thread.sleep(time);
        return false;
      }
      catch(InterruptedException ex) {
        return true;
      }
    }
    
    public boolean call_sleep(StateTarget test, long time) {
      if(test != null)
        _lastTest = test;
      _lastTest.ping();
      return test.sleep(time);
    }
    
    public void call_ping(StateTarget test) {
      if(test != null)
        _lastTest = test;
      _lastTest.ping();
    } 
    
    private StateTarget _lastTest;
	}
  
  static public void main(String args[])
	{
		junit.textui.TestRunner.run(new TestSuite(StateTest.class));
	}
}