/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Intalio Inc.  For written permission, please
 *    contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Intalio Inc. Exolab is a registered trademark of
 *    Intalio Inc.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY INTALIO AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL INTALIO OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2000 (C) Intalio Inc. All Rights Reserved.
 *
 * $Id: ValuetypeTest.java,v 1.5 2001/08/13 03:47:35 wood Exp $
 *
 * Date         Author  Changes
 */

package org.openorb.test.iiop.value;

import org.omg.CORBA.Policy;
import org.omg.PortableServer.*;
import org.openorb.test.ORBTestCase;
import junit.framework.TestSuite;

import java.util.Arrays;

/**
 * Tests marshaling and unmarshaling of various valuetypes.
 *
 * @version $Revision: 1.5 $ $Date: 2001/08/13 03:47:35 $ 
 * @author <a href="mailto:wood@intalio.com">Chris Wood &lt;wood@intalio.com&gt;</a>
 * @see     org.exolab.jtf.CWTestCateogry
 * @see     org.exolab.jtf.CWTestCase
 */
public class ValuetypeTest extends ORBTestCase
{
	public ValuetypeTest(String name) {
		super(name);
	}
	
	protected void setUp() {
		super.setUp();
		
    try {
      // find the root poa
      org.omg.CORBA.ORB orb = getORB();
      _orb = (org.omg.CORBA_2_3.ORB)orb;

      POA rootPOA = (POA)orb.resolve_initial_references("RootPOA");

      rootPOA.the_POAManager().activate();

      TestTC01 svr_ref = (new TestTC01Impl(rootPOA))._this(orb);

      _cltRef = TestTC01Helper.narrow(forceMarshal(svr_ref));
      
      _any = _orb.create_any();
    }
    catch(org.omg.CORBA.UserException ex) {
      fail("exception during setup:" + ex.toString());
    }
	}

	private org.omg.CORBA_2_3.ORB _orb;
	private TestTC01 _cltRef;
  private org.omg.CORBA.Any _any;
	
	
  /**
	  * Test the Basic valuetype.
   */
	public void testBasicValuetype() {
		Object ret;
		
		ret = _cltRef.get_value_a(0);
		assertTrue("Value returned from get_value_a(0) is not a ValueA",
			ret instanceof ValueA);
		
		ret = _cltRef.get_value(0);
		assertTrue("Value returned from get_value(0) is not a ValueA",
			ret instanceof ValueA);

		ret = _cltRef.get_abstract(0);
		assertTrue("Value returned from get_abstract(0) is not a ValueA",
			ret instanceof ValueA);
    
    ValueAHelper.insert(_any, (ValueA)ret);
    _cltRef.echo_any(_any);
	}
	
	/**
	 * Test Valuetype with private members.
	 */
	public void testPMValuetype() {
		Object ret;

		ret = _cltRef.get_value_b();
		assertTrue("Value returned from get_value_b() is not a ValueB",
			ret instanceof ValueB);
		
		ret = _cltRef.get_value(1);
		assertTrue("Value returned from get_value(1) is not a ValueB",
			ret instanceof ValueB);

		ret = _cltRef.get_abstract(1);
		assertTrue("Value returned from get_abstract(1) is not a ValueB",
			ret instanceof ValueB);
	}
	
	/**
	 * Test Nontruncatable, requiring factory.
	 */
	public void testNontruncatableValuetype() {
		try {
			// this should fail, no factory registered.
			_cltRef.get_value_c();
			fail("Read a ValueC without factory (should fail)");
		}
		catch(org.omg.CORBA.SystemException ex) {}

		try {
			// this should fail, no factory registered.
			_cltRef.get_value_a(2);
			fail("Read a ValueC as ValueA without factory (should fail)");
		}
		catch(org.omg.CORBA.SystemException ex) {}

		try {
			// this should fail, no factory registered.
			_cltRef.get_value(2);
			fail("Read a ValueC as ValueBase without factory (should fail)");
		}
		catch(org.omg.CORBA.SystemException ex) {}

		_orb.register_value_factory(ValueCHelper.id(), new ValueCFactory());

		Object ret;

		ret = _cltRef.get_value_c();
		assertTrue("Value returned from get_value_c() is not a ValueC",
			ret instanceof ValueC);
		
		ret = _cltRef.get_value_a(2);
		assertTrue("Value returned from get_value_a(2) is not a ValueC",
			ret instanceof ValueC);
		
		ret = _cltRef.get_value(2);
		assertTrue("Value returned from get_value(2) is not a ValueC",
			ret instanceof ValueC);

		ret = _cltRef.get_abstract(2);
		assertTrue("Value returned from get_abstract(2) is not a ValueC",
			ret instanceof ValueC);
	}
  
	/**
	 * Test Truncatable, requiring factory.
	 */
	public void testTruncatableValuetype() {
		try {
			// this should fail, no factory registered.
			_cltRef.get_value_d();
			fail("Read a ValueD without factory (should fail)");
		}
		catch(org.omg.CORBA.SystemException ex) {}

		Object ret;

		ret = _cltRef.get_value_a(3);
		assertTrue("Truncated value returned from get_value_a(3) is not a ValueA",
			ret instanceof ValueA);
		assertTrue("Truncated value returned from get_value_a(3) is a ValueD",
			!(ret instanceof ValueD));
		
		ret = _cltRef.get_value(3);
		assertTrue("Truncated value returned from get_value(3) is not a ValueA",
			ret instanceof ValueA);
		assertTrue("Truncated value returned from get_value(3) is a ValueD",
			!(ret instanceof ValueD));

		_orb.register_value_factory(ValueDHelper.id(), new ValueDFactory());

		ret = _cltRef.get_value_d();
		assertTrue("Value returned from get_value_d() is not a ValueD",
			ret instanceof ValueD);
		
		ret = _cltRef.get_value_a(3);
		assertTrue("Value returned from get_value_a(3) is not a ValueD",
			ret instanceof ValueD);
		
		ret = _cltRef.get_value(3);
		assertTrue("Value returned from get_value(3) is not a ValueD",
			ret instanceof ValueD);

		ret = _cltRef.get_abstract(3);
		assertTrue("Value returned from get_abstract(3) is not a ValueC",
			ret instanceof ValueD);
	}
  
  /**
   * Test Supports valuetype.
   */
  public void testSupportsValuetype() {
		Object ret;

		ret = _cltRef.get_value_e();
		assertTrue("Value returned from get_value_e() is not a ValueERemoteOperations",
			ret instanceof ValueERemoteOperations);
		assertTrue("Value returned from get_value_e() is not a ValueE",
			ret instanceof ValueE);
		assertTrue("Value returned from get_value_e() is an object reference",
			!(ret instanceof org.omg.CORBA.Object));
		
		ret = _cltRef.get_value_e_remote();
		assertTrue("Value returned from get_value_e_remote() is not a ValueERemote",
			ret instanceof ValueERemote);
		assertTrue("Value returned from get_value_e_remote() is not an object reference",
			ret instanceof org.omg.CORBA.Object);

    ret = _cltRef.get_value(4);
		assertTrue("Value returned from get_value(4) is not a ValueE",
			ret instanceof ValueE);

		ret = _cltRef.get_abstract(4);
		assertTrue("Value returned from get_abstract(4) is not a ValueE",
			ret instanceof ValueE);
  }

  /**
   * Test custom marshal.
   */
  public void testCustomMarshalValuetype() {
		Object ret;

		ret = _cltRef.get_value_f();
		assertTrue("Value returned from get_value_f() is not a ValueF",
			ret instanceof ValueF);
		
		ret = _cltRef.get_value(5);
		assertTrue("Value returned from get_value(5) is not a ValueF",
			ret instanceof ValueF);

		ret = _cltRef.get_abstract(5);
		assertTrue("Value returned from get_abstract(5) is not a ValueF",
			ret instanceof ValueF);
  }
    
  /**
   * Test custom marshal.
   */
  public void testNestedValuetype() {
		Object ret;

		ret = _cltRef.get_value_g();
		assertTrue("Value returned from get_value_g() is not a ValueG",
			ret instanceof ValueG);
		
		ret = _cltRef.get_value(6);
		assertTrue("Value returned from get_value(6) is not a ValueG",
			ret instanceof ValueG);

		ret = _cltRef.get_abstract(6);
		assertTrue("Value returned from get_abstract(6) is not a ValueG",
			ret instanceof ValueG);
    
    ValueG val = (ValueG)ret;
    
    assertTrue("Valuetype cycle has not transmitted properly",
       val == val.left.parent && val == val.right.parent);
  }
  
  /**
   * Test abstract valuetypes.
   */
  public void testAbstract() {
		Object ret;
    
    ret = _cltRef.get_abstract_a(0);
		assertTrue("Value returned from get_abstract_a(0) is not an AbstractA",
			ret instanceof AbstractA);
		assertTrue("Value returned from get_abstract_a(0) is not an Object",
			ret instanceof org.omg.CORBA.Object);
    
    ret = _cltRef.get_abstract_a(1);
		assertTrue("Value returned from get_abstract_a(1) is not an AbstractA",
			ret instanceof AbstractA);
		assertTrue("Value returned from get_abstract_a(1) is an Object",
			!(ret instanceof org.omg.CORBA.Object));
    
    ret = _cltRef.get_abstract_a(2);
		assertTrue("Value returned from get_abstract_a(2) is not an AbstractA",
			ret instanceof AbstractA);
		assertTrue("Value returned from get_abstract_a(2) is an Object",
			!(ret instanceof org.omg.CORBA.Object));
    
    ret = _cltRef.get_abstract_a(3);
		assertTrue("Value returned from get_abstract_a(3) is not an AbstractA",
			ret instanceof AbstractA);
		assertTrue("Value returned from get_abstract_a(3) is not an Object",
			ret instanceof org.omg.CORBA.Object);    
    
    AbstractAHelper.insert(_any, (AbstractA)ret);
    _cltRef.echo_any(_any);
  }
	
	private static final int F_L = 49875;
	
	static class TestTC01Impl extends TestTC01POA {
		TestTC01Impl(POA poa) {
      _poa = poa;
    }

    private POA _poa;
    
    public POA _default_POA() {
      return _poa;
    }

		public org.omg.CORBA.Any echo_any(org.omg.CORBA.Any val) {
			return val;
		}

		public org.omg.CORBA.TypeCode echo_TypeCode(org.omg.CORBA.TypeCode arg) {
			return arg;
		}

		public LongBox echo_longBox(LongBox val) {
			return val;
		}

		public int[] echo_longBoxSeq(int[] val) {
			return val;
		}

		public java.io.Serializable get_value(int type) {
			switch(type) {
			case 0:
				return get_value_a(0);
			case 1:
				return get_value_b();
			case 2:
				return get_value_c();
			case 3:
				return get_value_d();
			case 4:
				return get_value_e();
			case 5:
				return get_value_f();
			case 6:
				return get_value_g();
			default:
				return null;
			}
		}

		public ValueA get_value_a(int type) {
			switch(type) {
			case 0:
				ValueA ret = new ValueAImpl();
				ret.l = 1;
				ret.str = "ValueA";
				return ret;
			case 2:
				return get_value_c();
			case 3:
				return get_value_d();
			default:
				return null;
			}
		}
		
		public ValueB get_value_b() {
			org.omg.CORBA_2_3.ORB orb = (org.omg.CORBA_2_3.ORB)_orb();			
			ValueBValueFactory factory = (ValueBValueFactory)orb.lookup_value_factory(ValueBHelper.id());
			
			ValueB ret = factory.init(1);
			ret.str = "ValueB";
			return ret;
		}

		public ValueC get_value_c() {
			ValueC ret = new ValueCImpl();
			ret.l = 1;
			ret.n = 0;
			ret.str = "ValueC";
			return ret;
		}
		
		public ValueD get_value_d() {
			ValueD ret = new ValueDImpl();
			ret.l = 1;
			ret.n = 0;
			ret.str = "ValueD";
			return ret;
		}

		public ValueERemote get_value_e_remote() {
			return (new ValueERemotePOATie(get_value_e(), _poa()))._this(_orb());
		}

		public ValueE get_value_e() {
			ValueE ret = new ValueEImpl("remote");
			ret.prefix = "ValueE";
			return ret;
		}

		public ValueF get_value_f() {
			ValueF ret = new ValueFImpl();
			ret.l = F_L;
			return ret;
		}

		public ValueG get_value_g() {
			ValueG root = new ValueGImpl();
			root.idx = 0;
			root.name = "root";
			root.left = new ValueGImpl();
			root.left.idx = 1;
			root.left.name = "left";
			root.left.parent = root;
			root.right = new ValueGImpl();
			root.right.idx = 1;
			root.right.name = "right";
			root.right.parent = root;
			return root;
		}

		public Object get_abstract(int type) {
			if(type < 7)
				return get_value(type);
			return get_abstract_a(type-7);
		}			

		public AbstractA get_abstract_a(int type) {
			AbstractA ret;
			switch(type) {
			case 0:
				return (new AbstractA1Impl(_poa()))._this(_orb());
			case 1:
				return new AbstractA2Impl();
			case 2:
				return new AbstractA3Impl("remote");
			case 3:
				return (new AbstractA1POATie(new AbstractA3Impl("remote"), _poa()))._this(_orb());
			default:
				return null;
			}
		}
	}
  
  static public void main(String args[])
	{
		junit.textui.TestRunner.run(new TestSuite(ValuetypeTest.class));
	}
}