/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Intalio Inc.  For written permission, please
 *    contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Intalio Inc. Exolab is a registered trademark of
 *    Intalio Inc.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY INTALIO AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL INTALIO OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2000 (C) Intalio Inc. All Rights Reserved.
 *
 * $Id: NamingTest.java,v 1.4 2001/08/13 03:47:35 wood Exp $
 *
 * Date         Author  Changes
 */

package org.openorb.test.naming;

import junit.framework.TestSuite;
import org.openorb.test.ORBTestCase;
import org.openorb.util.NamingUtils;
import org.openorb.util.MapNamingContext;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.InvalidName;
import org.omg.CosNaming.NamingContextPackage.NotFound;
import org.omg.PortableServer.POA;

/**
 * Tests naming service.
 * <p>
 *
 * @version $Revision: 1.4 $ $Date: 2001/08/13 03:47:35 $ 
 * @author <a href="mailto:wood@intalio.com">Chris Wood &lt;wood@intalio.com&gt;</a>
 */
public class NamingTest extends ORBTestCase
{
  public NamingTest(String name) {
    super(name);
  }

  private static boolean compareNames(NameComponent [] nc1, NameComponent [] nc2) {
    if(nc1.length != nc2.length)
      return false;
    for(int i = 0; i < nc1.length; ++i)
      if(!nc1[i].id.equals(nc2[i].id) ||
         !nc1[i].kind.equals(nc2[i].kind))
        return false;
    return true;
  }
  
  /**
   * This test apply several Name creations and transformations
   *
   * @exception org.omg.CosNaming.NamingContextPackage.InvalidName if any of the test cases fails
   */
  public void testNameManipulation() throws InvalidName {
    NameComponent [] n;
    String s, debug;

    debug = "Empty name";
    n = new NameComponent[1];
    n[0] = new NameComponent("","");
    s = NamingUtils.to_string(n);
    assertEquals(debug, s, ".");
    assertTrue(debug, compareNames(n, NamingUtils.to_name(s)));

    debug = ("id only");
    n = new NameComponent[1];
    n[0] = new NameComponent("hello","");
    s = NamingUtils.to_string(n);
    assertEquals(debug, s, "hello");
    assertTrue(debug, compareNames(n, NamingUtils.to_name(s)));

    debug = ("kind only");
    n = new NameComponent[1];
    n[0] = new NameComponent("","world");
    s = NamingUtils.to_string(n);
    assertEquals(debug, s, ".world");
    assertTrue(debug, compareNames(n, NamingUtils.to_name(s)));

    debug = ("both");
    n = new NameComponent[1];
    n[0] = new NameComponent("hello","world");
    s = NamingUtils.to_string(n);
    assertEquals(debug, s, "hello.world");
    assertTrue(debug, compareNames(n, NamingUtils.to_name(s)));

    debug = ("Multiple elements...");
    n = new NameComponent[4];
    n[0] = new NameComponent("","");
    n[1] = new NameComponent("hello","");
    n[2] = new NameComponent("","world");
    n[3] = new NameComponent("hello","world");
    s = NamingUtils.to_string(n);
    assertEquals(debug, s, "./hello/.world/hello.world");
    assertTrue(debug, compareNames(n, NamingUtils.to_name(s)));

    debug = ("names needing escaping");
    n = new NameComponent[1];
    n[0] = new NameComponent(".","");
    s = NamingUtils.to_string(n);
    assertEquals(debug, s, "\\.");
    assertTrue(debug, compareNames(n, NamingUtils.to_name(s)));

    n = new NameComponent[1];
    n[0] = new NameComponent("/","");
    s = NamingUtils.to_string(n);
    assertEquals(debug, s, "\\/");
    assertTrue(debug, compareNames(n, NamingUtils.to_name(s)));

    n = new NameComponent[1];
    n[0] = new NameComponent("\\","");
    s = NamingUtils.to_string(n);
    assertEquals(debug, s, "\\\\");
    assertTrue(debug, compareNames(n, NamingUtils.to_name(s)));

    n = new NameComponent[2];
    n[0] = new NameComponent("\\\\\\.../\\.","/.//.\\");
    n[1] = new NameComponent("m.o.r.e.","//slashy\\\\");
    s = NamingUtils.to_string(n);
    assertEquals(debug, s, "\\\\\\\\\\\\\\.\\.\\.\\/\\\\\\..\\/\\.\\/\\/\\.\\\\/m\\.o\\.r\\.e\\..\\/\\/slashy\\\\\\\\");
    assertTrue(debug, compareNames(n, NamingUtils.to_name(s)));

    s = "";
    try { NamingUtils.to_name(s); fail("Exception should be thrown"); }
    catch(InvalidName ex) { }

    s = "..";
    try { NamingUtils.to_name(s); fail("Exception should be thrown"); }
    catch(InvalidName ex) { }

    s = "hello.world/";
    try { NamingUtils.to_name(s); fail("Exception should be thrown"); }
    catch(InvalidName ex) { }

    s = "I.want.blood";
    try { NamingUtils.to_name(s); fail("Exception should be thrown"); }
    catch(InvalidName ex) { }
  }
  
	/**
   * Test name service.
   *
   * @exception org.omg.CORBA.UserException if any of the test case fails
   */
  public void testNameService() throws org.omg.CORBA.UserException {
    // find the root poa
    org.omg.CORBA.ORB orb = getORB();

    POA rootPOA = (POA)orb.resolve_initial_references("RootPOA");
    rootPOA.the_POAManager().activate();

    MapNamingContext map = new MapNamingContext(orb, null);

    NamingContextExt ctxt = map.getRootCtxt();

    map.put("empty", null);
    map.put("submap.context/recurse", ctxt);
    map.put("recurse", ctxt);			

    NameComponent [] n;
    org.omg.CORBA.Object res = null;
    org.omg.CORBA.Object sub = null;
    BindingListHolder bl;
    BindingIteratorHolder bi;
    BindingHolder bh;

    bl = new BindingListHolder();
    bi = new BindingIteratorHolder();
    ctxt.list(1, bl, bi);

    assertTrue(bl.value.length == 1);
    if(bi.value != null) {
      bh = new BindingHolder();
      while(bi.value.next_one(bh))
        ; // do nothing
      bi.value.destroy();
    }

    n = new NameComponent[1];
    n[0] = new NameComponent("empty", "");
    res = ctxt.resolve(n);
    assertNull(res);

    n = new NameComponent[1];
    n[0] = new NameComponent("recurse", "");		
    res = ctxt.resolve(n);
    assertTrue(res._is_equivalent(ctxt));

    sub = null;
    n[0] = new NameComponent("submap", "context");
    sub = ctxt.resolve(n);
    assertTrue(!sub._is_equivalent(ctxt));

    res = null;
    n[0] = new NameComponent("recurse", "");
    res = NamingContextHelper.narrow(sub).resolve(n);
    assertTrue(res._is_equivalent(ctxt));

    n = new NameComponent[2];
    n[0] = new NameComponent("submap", "context");		
    n[1] = new NameComponent("recurse", "");		
    res = ctxt.resolve(n);
    assertTrue(res._is_equivalent(ctxt));

    sub = null;
    n[0] = new NameComponent("recurse", "");		
    n[1] = new NameComponent("submap", "context");		
    sub = ctxt.resolve(n);
    assertTrue(!sub._is_equivalent(ctxt));

    sub = null;
    n[0] = new NameComponent("recurse", "");		
    n[1] = new NameComponent("nonexist", "");		
    try {
      sub = ctxt.resolve(n);
      fail("Able to resolve nonexistent context");
    }
    catch (NotFound ex) {
      // expected
    }
  }  
  
  static public void main(String args[])
	{
		junit.textui.TestRunner.run(new TestSuite(NamingTest.class));
	}
}