/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Intalio Inc.  For written permission, please
 *    contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Intalio Inc. Exolab is a registered trademark of
 *    Intalio Inc.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY INTALIO AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL INTALIO OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2000 (C) Intalio Inc. All Rights Reserved.
 *
 * $Id: PITest.java,v 1.8 2001/08/13 03:47:35 wood Exp $
 *
 * Date         Author  Changes
 */

package org.openorb.test.pi;

import junit.framework.TestSuite;
import org.omg.CORBA.Policy;
import org.omg.PortableServer.*;
import org.openorb.test.ORBTestCase;

import java.util.Properties;
import org.openorb.test.adapter.poa.Hello;
import org.openorb.test.adapter.poa.HelloPOA;
import org.openorb.test.adapter.poa.HelloHelper;

import java.io.IOException;

/**
 * Tests marshaling and unmarshaling of various iiop types.
 * <p>
 *
 * @version $Revision: 1.8 $ $Date: 2001/08/13 03:47:35 $ 
 * @author <a href="mailto:wood@intalio.com">Chris Wood &lt;wood@intalio.com&gt;</a>
 * @see     org.exolab.jtf.CWTestCateogry
 * @see     org.exolab.jtf.CWTestCase
 */
public class PITest extends ORBTestCase
{
  /**
   * The constructor is responsible for constructing a test category and
   * adding the suite of test cases. It throws CWClassConstructorException
   * if it cannot construct the category
   *
   * @exception   Exception
   */
  public PITest(String name)
  {
    super(name);
    
    __visitMask = 0;
    __throwExcept = 0;
    __retryCount = 0;
  }
	
	protected void setUp()
	{
		java.util.Properties props = new java.util.Properties();
		// set known iiop port since persistant references are used.
    props.setProperty("org.omg.PortableInterceptor.ORBInitializerClass.Empty", EmptyInitializer.class.getName());
		setUp(props);
    
    try {
			// find the root poa
			_orb = getORB();

      POA rootPOA = (POA)_orb.resolve_initial_references("RootPOA");

      Hello svrRef = (new HelloImpl(rootPOA))._this(_orb);

			rootPOA.the_POAManager().activate();

      _cltRef = HelloHelper.narrow(forceMarshal(svrRef));
      
      __any = _orb.create_any();
      __any.insert_boolean(true);
		}
		catch(org.omg.CORBA.UserException ex) {
			fail("exception during setup:" + ex.toString());
		}
	}

  private Hello _cltRef = null;
  
  private static final int SEND_REQ = 0x1;
  private static final int SEND_POLL = 0x2;
  private static final int SEND_SC = 0x8;
  private static final int RECV_REQSC = 0x10;
  private static final int RECV_REQ = 0x20;
  private static final int RECV_SC = 0x80;
  private static final int SEND_REPL = 0x100;
  private static final int SEND_EXPT = 0x200;
  private static final int SEND_OTHR = 0x400;
  private static final int SEND_REPL_SC = 0x800;
  private static final int RECV_REPL = 0x1000;
  private static final int RECV_EXPT = 0x2000;
  private static final int RECV_OTHR = 0x4000;
  private static final int RECV_REPL_SC = 0x8000;
  
  private static final int NOR_PATH = SEND_REQ | RECV_REQSC | RECV_REQ | SEND_REPL | RECV_REPL;
  private static final int NOR_SC = SEND_SC | RECV_SC | SEND_REPL_SC | RECV_REPL_SC;
  
  private static final int TEST_SCID = 0x444f7F01;
  
  private static int __throwExcept;
  private static int __visitMask;
  private static int __retryCount;
  
  private static int __slotID;
  private static org.omg.CORBA.Any __any;
  
  private org.omg.CORBA.ORB _orb;
  
  /**
   * Test complete request call.
   */
  public void testCompleteCall() {
    __throwExcept = 0;
    try {
      _cltRef.hello("");
    }
    catch(org.omg.CORBA.UNKNOWN ex) {
      fail("unexpected exception received");
    }
    assertEquals("Complete call did not visit all interception points.", __visitMask, NOR_PATH);
  }
  
  /**
   * Test complete request call with service contexts.
   *
   * @exception org.omg.CORBA.UserException if any of the test cases fails
   */
  public void testCompleteCallWithSCs() throws org.omg.CORBA.UserException {
    __throwExcept = 0;
    org.omg.PortableInterceptor.Current curr =
      (org.omg.PortableInterceptor.Current)_orb.resolve_initial_references("PICurrent");

    curr.set_slot(__slotID, __any);  
    
    try {
      _cltRef.hello("");
    }
    catch(org.omg.CORBA.UNKNOWN ex) {
      fail("unexpected exception received");
    }
    assertEquals("Complete call did not visit all interception points.", __visitMask, NOR_PATH | NOR_SC);
  }
  
  /**
   * Abort at send_request
   */
  public void testToSendRequest() {
    __throwExcept = SEND_REQ;
    try {
      _cltRef.hello("");
    }
    catch(org.omg.CORBA.UNKNOWN ex) {
      assertEquals("Complete call did not visit all interception points.", __visitMask, SEND_REQ);
      assertEquals("Exception thrown in wrong place", ex.minor, SEND_REQ);
      return;
    }
    fail("expected exception");
  }
  
  /**
   * Abort at receive_request_service_contexts
   */
  public void testToRecvRequestSC() {
    __throwExcept = RECV_REQSC;
    try {
      _cltRef.hello("");
    }
    catch(org.omg.CORBA.UNKNOWN ex) {
      assertEquals("Complete call did not visit all interception points.", 
        __visitMask, SEND_REQ | RECV_REQSC | RECV_EXPT);
      assertEquals("Exception thrown in wrong place", ex.minor, RECV_REQSC);
      return;
    }
    fail("expected exception");
  }
  
  /**
   * Abort at receive_request_service_contexts and recieve_exception
   */
  public void testToRecvRequestSCReceiveException() {
    __throwExcept = RECV_REQSC | RECV_EXPT;
    try {
      _cltRef.hello("");
    }
    catch(org.omg.CORBA.UNKNOWN ex) {
      assertEquals("Complete call did not visit all interception points.", 
        __visitMask, SEND_REQ | RECV_REQSC | RECV_EXPT);
      assertEquals("Exception thrown in wrong place", ex.minor, RECV_REQSC | RECV_EXPT);
      return;
    }
    fail("expected exception");
  }
  
  /**
   * Abort at receive_request
   */
  public void testToRecvRequest() {
    __throwExcept = RECV_REQ;
    try {
      _cltRef.hello("");
    }
    catch(org.omg.CORBA.UNKNOWN ex) {
      assertEquals("Complete call did not visit all interception points.", 
        __visitMask, SEND_REQ | RECV_REQSC | RECV_REQ | RECV_EXPT);
      assertEquals("Exception thrown in wrong place", ex.minor, RECV_REQ);
      return;
    }
    fail("expected exception");
  }
  
  /**
   * Abort at receive_request and recieve_exception
   */
  public void testToRecvRequestRecvExcept() {
    __throwExcept = RECV_REQ | RECV_EXPT;
    try {
      _cltRef.hello("");
    }
    catch(org.omg.CORBA.UNKNOWN ex) {
      assertEquals("Complete call did not visit all interception points.", 
        __visitMask, SEND_REQ | RECV_REQSC | RECV_REQ | RECV_EXPT);
      assertEquals("Exception thrown in wrong place", ex.minor, RECV_REQ | RECV_EXPT);
      return;
    }
    fail("expected exception");
  }
  
  /**
   * Abort at send_reply
   */
  public void testToSendReply() {
    __throwExcept = SEND_REPL;
    try {
      _cltRef.hello("");
    }
    catch(org.omg.CORBA.UNKNOWN ex) {
      assertEquals("Complete call did not visit all interception points.", 
        __visitMask, SEND_REQ | RECV_REQSC | RECV_REQ | SEND_REPL | RECV_EXPT);
      assertEquals("Exception thrown in wrong place", ex.minor, SEND_REPL);
      return;
    }
    fail("expected exception");
  }
  
  /**
   * Abort at send_reply
   */
  public void testToRecvReply() {
    __throwExcept = RECV_REPL;
    try {
      _cltRef.hello("");
    }
    catch(org.omg.CORBA.UNKNOWN ex) {
      assertEquals("Complete call did not visit all interception points.", 
        __visitMask, NOR_PATH);
      assertEquals("Exception thrown in wrong place", ex.minor, RECV_REPL);
      return;
    }
    fail("expected exception");
  }
	
	static class HelloImpl 
    extends HelloPOA
  {
    HelloImpl(POA poa) {
      _poa = poa;
    }

    private POA _poa;
    
    public void hello(String msg) {
    }
    
    public POA _default_POA() {
      return _poa;
    }
  }

	public static class EmptyInitializer 
		extends org.omg.CORBA.LocalObject
		implements org.omg.PortableInterceptor.ORBInitializer
	{
		
		public void pre_init(org.omg.PortableInterceptor.ORBInitInfo info) {
			try {
        __slotID = info.allocate_slot_id();
				info.add_server_request_interceptor(new EmptyServerInterceptor());
				info.add_client_request_interceptor(new EmptyClientInterceptor());
        info.add_ior_interceptor(new EmptyIORInterceptor());
			}
			catch(org.omg.PortableInterceptor.ORBInitInfoPackage.DuplicateName ex) {}
		}
		
		public void post_init(org.omg.PortableInterceptor.ORBInitInfo info) {
		}
	}
	
  static class EmptyIORInterceptor 
    extends org.omg.CORBA.LocalObject
    implements org.omg.PortableInterceptor.IORInterceptor
  {
    public String name() {
      return "";
    }    
    
    public void destroy() {}
    
    public void establish_components(org.omg.PortableInterceptor.IORInfo info) {
      info.add_ior_component(new org.omg.IOP.TaggedComponent(TEST_SCID, new byte[0]));
    }      
  }
  
	static class EmptyClientInterceptor
		extends org.omg.CORBA.LocalObject
		implements org.omg.PortableInterceptor.ClientRequestInterceptor
	{
		public String name() {
			return "";
		}

    public void destroy() {}
    
		public void send_request(org.omg.PortableInterceptor.ClientRequestInfo ri)
			throws org.omg.PortableInterceptor.ForwardRequest 
		{
      if((__visitMask & SEND_REQ) != 0)
        __retryCount++;
      
      __visitMask = SEND_REQ;
      if(__throwExcept == SEND_REQ)
				throw new org.omg.CORBA.UNKNOWN(SEND_REQ, org.omg.CORBA.CompletionStatus.COMPLETED_NO);
      
      // add service context.
      try {
        if(ri.get_slot(__slotID).type().kind() != org.omg.CORBA.TCKind.tk_null) {    
          ri.add_request_service_context(new org.omg.IOP.ServiceContext(TEST_SCID, new byte[0]), true);
          __visitMask = __visitMask | SEND_SC;
        }
      }
      catch(org.omg.PortableInterceptor.InvalidSlot ex) {
        fail(ex.toString());
      }
      
      // request information.
      ri.request_id();
      assertEquals("Operation name not correct", ri.operation(), "hello");
      assertTrue("No response expected for request with response", ri.response_expected());
      assertEquals("Incorrect sync scope", ri.sync_scope(), 3);
      
      // target information
      ri.target();
      ri.effective_target();
      ri.effective_profile();
      ri.get_effective_component(org.omg.IOP.TAG_CODE_SETS.value);
      ri.get_effective_components(org.omg.IOP.TAG_CODE_SETS.value);
    
      // test retrieving request parameters. These will always fail
      try {
        ri.arguments();
      }
      catch(org.omg.CORBA.NO_RESOURCES ex) {}
      try {
        ri.exceptions();
      }
      catch(org.omg.CORBA.NO_RESOURCES ex) {}
      try {
        ri.contexts();
      }
      catch(org.omg.CORBA.NO_RESOURCES ex) {}
      try {
        ri.operation_context();
      }
      catch(org.omg.CORBA.NO_RESOURCES ex) {}


      /*
        forward_reference no no no no yes 2
        get_slot yes yes yes yes yes
        get_request_service_context yes no yes yes yes
        get_reply_service_context no no yes yes yes
        get_request_policy yes no yes yes yes
        add_request_service_context yes no no no no
       */
		}
		
		public void send_poll(org.omg.PortableInterceptor.ClientRequestInfo ri) {
      __visitMask = __visitMask | SEND_POLL;
      if((__throwExcept & SEND_POLL) != 0)
				throw new org.omg.CORBA.UNKNOWN(SEND_POLL, org.omg.CORBA.CompletionStatus.COMPLETED_YES);        
		}

    public void receive_reply(org.omg.PortableInterceptor.ClientRequestInfo ri) {
      __visitMask = __visitMask | RECV_REPL;
      if(0 != (__throwExcept & RECV_REPL))
				throw new org.omg.CORBA.UNKNOWN(RECV_REPL, org.omg.CORBA.CompletionStatus.COMPLETED_YES);
      
      try {
        ri.get_reply_service_context(TEST_SCID);
        
        __visitMask = __visitMask | RECV_REPL_SC;
      }
      catch(org.omg.CORBA.BAD_PARAM ex) {}
      
      // test retrieving request results. These will always fail
      try {
        ri.result();
      }
      catch(org.omg.CORBA.NO_RESOURCES ex) {}
		}
		
		public void receive_other(org.omg.PortableInterceptor.ClientRequestInfo ri)
			throws org.omg.PortableInterceptor.ForwardRequest 
		{
      __visitMask = __visitMask | RECV_OTHR;
      if(0 != (__throwExcept & RECV_OTHR))
				throw new org.omg.CORBA.UNKNOWN(RECV_OTHR, org.omg.CORBA.CompletionStatus.COMPLETED_YES);
		}
		
		public void receive_exception(org.omg.PortableInterceptor.ClientRequestInfo ri)
			throws org.omg.PortableInterceptor.ForwardRequest 
		{
      __visitMask = __visitMask | RECV_EXPT;
      if(ri.received_exception_id().equals(org.omg.CORBA.UNKNOWNHelper.id())
          && 0 != (__throwExcept & RECV_EXPT)) {
        org.omg.CORBA.UNKNOWN uex = org.omg.CORBA.UNKNOWNHelper.extract(ri.received_exception());
        uex.minor = uex.minor | RECV_EXPT;
        throw uex;
      }
 		}
		
	}
	
	static class EmptyServerInterceptor
		extends org.omg.CORBA.LocalObject
		implements org.omg.PortableInterceptor.ServerRequestInterceptor
	{
		int i = 0;
    
		public java.lang.String name() {
			return "";
		}
		
    public void destroy() {}
    
		public void receive_request_service_contexts(org.omg.PortableInterceptor.ServerRequestInfo ri) 
			throws org.omg.PortableInterceptor.ForwardRequest 
		{
      __visitMask = __visitMask | RECV_REQSC;
      if(0 != (__throwExcept & RECV_REQSC))
				throw new org.omg.CORBA.UNKNOWN(RECV_REQSC, org.omg.CORBA.CompletionStatus.COMPLETED_NO);
      
      // request information.
      ri.request_id();
      assertEquals("Operation name not correct", ri.operation(), "hello");
      assertTrue("No response expected for request with response", ri.response_expected());
      assertEquals("Incorrect sync scope", ri.sync_scope(), 3);

      try {
        ri.get_request_service_context(TEST_SCID);
        
        __visitMask = __visitMask | RECV_SC;
        
        ri.set_slot(__slotID, __any);
      }
      catch(org.omg.CORBA.BAD_PARAM ex) {}
      catch(org.omg.PortableInterceptor.InvalidSlot ex) {
        fail(ex.toString());
      }
      
      /*
reply_status no no yes yes yes
forward_reference no no no no yes 2
get_slot yes yes yes yes yes
get_request_service_context yes yes yes yes yes
get_reply_service_context no no yes yes yes
get_server_policy yes yes yes yes yes
set_slot yes yes yes yes yes
add_reply_service_context yes yes yes yes yes      
       */
		}
		
		public void receive_request(org.omg.PortableInterceptor.ServerRequestInfo ri)
			throws org.omg.PortableInterceptor.ForwardRequest 
		{
      __visitMask = __visitMask | RECV_REQ;
      if(0 != (__throwExcept & RECV_REQ))
				throw new org.omg.CORBA.UNKNOWN(RECV_REQ, org.omg.CORBA.CompletionStatus.COMPLETED_NO);

      // request information
      ri.object_id();
      ri.adapter_id();
      ri.target_most_derived_interface();
      assertTrue("target does not implement object", ri.target_is_a("IDL:omg.org/CORBA/Object:1.0"));
      
      
      // test retrieving request parameters. These will always fail
      try {
        ri.arguments();
      }
      catch(org.omg.CORBA.NO_RESOURCES ex) {}
      try {
        ri.exceptions();
      }
      catch(org.omg.CORBA.NO_RESOURCES ex) {}
      try {
        ri.contexts();
      }
      catch(org.omg.CORBA.NO_RESOURCES ex) {}
      try {
        ri.operation_context();
      }
      catch(org.omg.CORBA.NO_RESOURCES ex) {}
		}
		
		public void send_reply(org.omg.PortableInterceptor.ServerRequestInfo ri) {
      __visitMask = __visitMask | SEND_REPL;
      if(0 != (__throwExcept & SEND_REPL))
				throw new org.omg.CORBA.UNKNOWN(SEND_REPL, org.omg.CORBA.CompletionStatus.COMPLETED_YES);
      
      try {
        if(ri.get_slot(__slotID).type().kind() != org.omg.CORBA.TCKind.tk_null) {
          ri.add_reply_service_context(new org.omg.IOP.ServiceContext(TEST_SCID, new byte[0]), true);
        
          __visitMask = __visitMask | SEND_REPL_SC;
        }
      }
      catch(org.omg.PortableInterceptor.InvalidSlot ex) {
        fail(ex.toString());
      }
		}
		
		public void send_exception(org.omg.PortableInterceptor.ServerRequestInfo ri)
			throws org.omg.PortableInterceptor.ForwardRequest 
		{
      __visitMask = __visitMask | SEND_EXPT;
      org.omg.CORBA.Any any = ri.sending_exception();
      if(any.type().equals(org.omg.CORBA.UNKNOWNHelper.type())
          && 0 != (__throwExcept & SEND_EXPT)) {
        org.omg.CORBA.UNKNOWN uex = org.omg.CORBA.UNKNOWNHelper.extract(ri.sending_exception());
        uex.minor = uex.minor | SEND_EXPT;
        uex.completed = org.omg.CORBA.CompletionStatus.COMPLETED_YES;
        throw uex;
      }
      
      // test retrieving request results. These will always fail
      try {
        ri.result();
      }
      catch(org.omg.CORBA.NO_RESOURCES ex) {}
		}
		
		public void send_other(org.omg.PortableInterceptor.ServerRequestInfo ri)
			throws org.omg.PortableInterceptor.ForwardRequest 
		{
      __visitMask = __visitMask | SEND_OTHR;
      if(0 != (__throwExcept & SEND_OTHR))
				throw new org.omg.CORBA.UNKNOWN(SEND_OTHR, org.omg.CORBA.CompletionStatus.COMPLETED_YES);        
		}		
	}
  
  static public void main(String args[])
	{
		junit.textui.TestRunner.run(new TestSuite(PITest.class));
	}
}